 

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
/**
* Opens a dialog box which requests for a state name and the accepting
* status of a new state. The state name is checked for validity using
* the {@link faeditor.FAChecker#isValidStateName isValidStateName()}
* before adding the state. An error occurs if the state name is not a
* valid identifier, or if another state in the current FA is already
* using the same state name (which is checked using {@link
* faeditor.FAChecker#isStateNameAvailable isStateNameAvailable()}). If
* successful, the state arguments are passed to the specified workspace
* panel, along with the command ADDSTATE.
*/

public class AddStateDialogBox extends JDialog implements ActionListener 
{
 //~ Instance variables ************************************************
 /** The parent component of this dialog box. */
 private FAEditor container;

 /** Cancel button. */
 private JButton jbtCancel;

 /** OK button. */
 private JButton jbtOk;

 /** Checkbox for accepting flag. */
 private JCheckBox jchkAccept;

 /** Textfield for state name. */
 private JTextField jtfStateName;

 /** The name for the new state. */
 private String name;

 /** The workspace panel for the current FA. */
 private WorkSpacePanel workSpacePanel;

 /** The accepting flag for the new state. */
 private boolean accepting;

 //~ Constructors ******************************************************
 /**
 * The sole constructor for this class.
 *
 * @param parent set the {@link #container container} of this dialog
 * box.
 * @param inputSpacePanel sets the current {@link #workSpacePanel
 * workSpacePanel}.
 */
 public AddStateDialogBox(JFrame parent, WorkSpacePanel inputSpacePanel) 
 {
  super(parent, true);
  workSpacePanel = inputSpacePanel;
  container = (FAEditor) parent;
  setTitle("Add State");
  setSize(250, 170);
  JPanel p1 = new JPanel();
  p1.setLayout(new FlowLayout());
  JLabel label1 = new JLabel("State Name");
  label1.setPreferredSize(new Dimension(100, 19));
  p1.add(label1);
  p1.add(jtfStateName = new JTextField(10));
  jtfStateName.setText("q" + inputSpacePanel.getFa().getStateCount());
  JPanel p2 = new JPanel();
  p2.setLayout(new FlowLayout());
  JLabel label2 = new JLabel("Accepting State?");
  label2.setPreferredSize(new Dimension(100, 19));
  p2.add(label2);
  p2.add(jchkAccept = new JCheckBox());
  JPanel p3 = new JPanel();
  p3.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 5));
  p3.add(p1);
  p3.add(p2);
  p3.setBorder(new TitledBorder("Enter Input"));
  JPanel pButtons = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
  pButtons.add(jbtOk = new JButton(" OK "));
  pButtons.add(jbtCancel = new JButton("Cancel"));
  getContentPane().setLayout(new BorderLayout());
  getContentPane().add(p3, BorderLayout.CENTER);
  getContentPane().add(pButtons, BorderLayout.SOUTH);
  jbtOk.addActionListener(this);
  jbtCancel.addActionListener(this);
  jchkAccept.addActionListener(this);
 }

 //~ Methods ***********************************************************
 /**
 * Determines the course of action on press of a button or a checkbox
 * in the dialog box. If the 'Accepting?' checkbox is clicked, the
 * new state's accept flag is changed. If the 'OK' button is
 * pressed, the state name is checked for validity and availability,
 * and if both are true, the state name and the state accept flag is
 * passed to the current {@link #workSpacePanel workspace} along with
 * an ADDSTATE command. On a 'Cancel', the dialog box is removed
 * without any changes.
 *
 * @param e the action event generated by this dialog box.
 */
 public void actionPerformed(ActionEvent e) 
 {
  if (e.getSource() instanceof JButton) 
  {
   if (e.getSource() == jbtOk) 
   {
    name = jtfStateName.getText();
    if ((FAChecker.isValidStateName(this, name))&& (FAChecker.isStateNameAvailable(this,workSpacePanel.getFa(),name, null))) 
    {
     workSpacePanel.setStateName(name);
     workSpacePanel.setCommand(WorkSpacePanel.ADDSTATE);
     this.setVisible(false);
     container.setStatus("Click on an empty space in the "+ "workspace to create the state...");
    }
   } 
   else if (e.getSource() == jbtCancel) 
   {
    this.setVisible(false); 
   } 
   workSpacePanel.repaint();
  }
  if (e.getSource() instanceof JCheckBox) 
  {
   if (jchkAccept.isSelected()) 
   {
    workSpacePanel.setAccepting(accepting = true);
   } 
   else 
   {
    workSpacePanel.setAccepting(accepting = false); 
   }
  }
 }
}
