 

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
* Opens a dialog box which requests for the input for the new transition.
* The transition input is checked for validity using {@link
* faeditor.FAChecker#isValidInput isValidInput()}. The transition
* input is sent to the workspace panel for adding to the new transition.
*/
public class AddTransitionDialogBox extends JDialog implements ActionListener 
{
 //~ Instance variables 

 /** The parent component of this dialog box. */
 private FAEditor container;

 /** Cancel button. */
 private JButton jbtCancel;

 /** OK button. */
 private JButton jbtOk;

 /** Textfield for transition input. */
 private JTextField jtfInput;

 /** The workspace panel for the current FA. */
 private WorkSpacePanel workSpacePanel;

 //~ Constructors ******************************************************
 /**
 * The sole constructor for this class.
 *
 * @param parent sets the {@link #container} variable.
 * @param inputSpacePanel sets the {@link #workSpacePanel} variable.
 *********************************************************************/ 

 public AddTransitionDialogBox(JFrame parent,WorkSpacePanel inputSpacePanel) 
 {
  super(parent, true);
  workSpacePanel = inputSpacePanel;
  container = (FAEditor) parent;
  setTitle("Add Transition");
  setSize(230, 130);
  JPanel p1 = new JPanel();
  p1.setLayout(new FlowLayout());
  p1.add(new JLabel("Input Symbol:"));
  p1.add(jtfInput = new JTextField(10));
  JPanel p2 = new JPanel();
  p2.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 2));
  p2.setBorder(new TitledBorder("Enter Input Symbol for Transition"));
  p2.add(p1);
  JPanel pButtons = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
  pButtons.add(jbtOk = new JButton(" OK "));
  pButtons.add(jbtCancel = new JButton("Cancel"));
  getContentPane().setLayout(new BorderLayout());
  getContentPane().add(p2, BorderLayout.CENTER);
  getContentPane().add(pButtons, BorderLayout.SOUTH);

//Register Listeners

  jbtOk.addActionListener(this);
  jbtCancel.addActionListener(this);
 }

//~ Methods ***********************************************************
/**
* Determines the course of action on press of a button in the dialog
* box. On a 'Cancel', the dialog box is removed without any further
* changes. On an 'OK' action, the input is checked for validity. If
* it is valid, the input is passed to the {@link #workSpacePanel}
* along with an ADDTRANSITION command.
*
* @param e the action event generated by the dialog box.
**********************************************************************/

 public void actionPerformed(ActionEvent e) 
 {
  if (e.getSource() instanceof JButton) 
  {
   if (e.getSource() == jbtOk) 
   {
    FA fa = workSpacePanel.getFa();
    State source = workSpacePanel.getSelectedState();
    if (FAChecker.isValidInput(this, fa, jtfInput.getText())) 
    {
     this.setVisible(false);
     workSpacePanel.setTransInput(jtfInput.getText());
     workSpacePanel.setCommand(WorkSpacePanel.ADDTRANSITION);
     container.setStatus("Select the source state " + "for this transition...");
    }
   } 
   else if (e.getSource() == jbtCancel) 
   {
    this.setVisible(false);
   }
   workSpacePanel.repaint();
  }
 }
}
