 

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
* Implements a dialog box for editing the input alphabet of the FA.
*/

public class EditInputAlphabetDialogBox extends JDialog implements ActionListener 
{

//~ Instance variables ************************************************
/** Cancel button. */
 private JButton jbtCancel;

/** OK button. */
 private JButton jbtOK;

/** Editable textfield containing the alphabet. */
 private JTextField newAlphabet;

/** Non-editable textfield containing the alphabet. */
 private JTextField oldAlphabet;

/** The edited input alphabet. */
 private String newAlpha;

/** The unedited input alphabet. */
 private String oldAlpha;

/**
* The workspace panel currently being used. The FA is obtained from
* this workspace panel.
*/
 private WorkSpacePanel workSpacePanel;

/** String arguments. */
 private String[] args = new String[2];

//~ Constructors ******************************************************
/**
* Generates the dialog box for this class.
*
* @param parent the parent component of this dialog box
* @param inputSpacePanel the workspace panel
*/

 public EditInputAlphabetDialogBox(JFrame parent,WorkSpacePanel inputSpacePanel) 
 {
  super(parent, true);
  workSpacePanel = inputSpacePanel;
  setTitle("Edit Input Alphabet");
  setSize(300, 170);
  JPanel p1 = new JPanel();
  p1.setLayout(new FlowLayout());
  JLabel label1 = new JLabel("Current Alphabet");
  label1.setPreferredSize(new Dimension(100, 19));
  p1.add(label1);
  p1.add(oldAlphabet = new JTextField(String.valueOf(workSpacePanel.getFa().getAlphabet()),10));
  oldAlphabet.setEditable(false);
  JPanel p2 = new JPanel();
  p2.setLayout(new FlowLayout());
  JLabel label2 = new JLabel("New Alphabet");
  label2.setPreferredSize(new Dimension(100, 19));
  p2.add(label2);
  p2.add(newAlphabet = new JTextField(String.valueOf(workSpacePanel.getFa().getAlphabet()),10));
  JPanel pMain = new JPanel();
  pMain.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 5));
  pMain.add(p1);
  pMain.add(p2);
  pMain.setBorder(new TitledBorder("Rename Alphabet"));
  JPanel pButtons = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
  pButtons.add(jbtOK = new JButton(" OK "));
  pButtons.add(jbtCancel = new JButton("Cancel"));
  getContentPane().setLayout(new BorderLayout());
  getContentPane().add(pMain, BorderLayout.CENTER);
  getContentPane().add(pButtons, BorderLayout.SOUTH);
  newAlphabet.addActionListener(this);
  jbtOK.addActionListener(this);
  jbtCancel.addActionListener(this);
 }

//~ Methods ***********************************************************
/**
* Implements the action for a button-click. On an OK, the edited
* alphabet is checked for validity. If it is valid, it is compared
* with the unedited alphabet for removed inputs. Transitions on
* these removed inputs are deleted from the FA.
*
* @param e the action event generated by a click on a button on the
* dialog box
*/

 public void actionPerformed(ActionEvent e) 
 {
  int result = FAChecker.OK;
  char[] newIA = FAChecker.expandAlphabetString(newAlphabet.getText());
  char[] oldIA = workSpacePanel.getFa().getAlphabet();
  if (e.getSource() instanceof JButton) 
  {
   if (e.getSource() == jbtOK) 
   {
    if (FAChecker.isValidInputAlphabet(this,newAlphabet.getText())) 
    {
     this.setVisible(false);
     args[0] = FAChecker.compareAlphabet(oldIA, newIA);
     if (args[0].length() > 0) 
     {
      result = FAChecker.displayWarning(this, 105);
     }
     if (result == FAChecker.OK) 
     {
      workSpacePanel.getFa().setAlphabet(newIA);
     }
    }
   } 
   else if (e.getSource() == jbtCancel) 
   {
    this.setVisible(false); 
   }
   workSpacePanel.repaint();
  }
 }
}
