 
 
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
* Implements a dialog box used for editing a state's parameters (i.e. the
* name and the accepting flag).
*/
public class EditStateDialogBox extends JDialog implements ActionListener 
{
 //~ Instance variables ************************************************
 /** The working FA. */
 private FA fa = null;

 /** Cancel button. */
 private JButton jbtCancel;
 
 /** OK button. */
 private JButton jbtOk;

 /** Checkbox for state accepting flag. */
 private JCheckBox jchkAccept;

 /** Checking for starting state assignment. */
 private JCheckBox jchkStart;
 
 /** Textfield for state name. */
 private JTextField jtfNewStateName;

 /** The state being edited. */
 private State state = null;

 /** The name of the state being edited. */
 private String name = "null";

 /** The current workspace panel. */
 private WorkSpacePanel workSpacePanel;

 /** The accepting flag of the state. */
 private boolean accepting;

 /** Checks if the state is the starting state. */
 private boolean start = false;
 
 //~ Constructors ******************************************************
 /**
 * Generates the dialog box for this class.
 *
 * @param parent the parent component of this dialog box
 * @param inputSpacePanel the workspace panel
 * @param state the state being edited
 */
 public EditStateDialogBox(JFrame parent,WorkSpacePanel inputSpacePanel, State state) 
 {
  super(parent, true);
  workSpacePanel = inputSpacePanel;
  setTitle("Edit State");
  setSize(300, 170);
  JPanel p1 = new JPanel();
  p1.setLayout(new FlowLayout());
  JLabel label1 = new JLabel("State Name");
  label1.setPreferredSize(new Dimension(100, 19));
  p1.add(label1);  
  p1.add(jtfNewStateName = new JTextField(state.getName(), 15));
  JPanel p2 = new JPanel();
  jchkAccept = new JCheckBox();
  accepting = state.getAcceptFlag();  
  jchkAccept.setSelected(accepting);
  p2.setLayout(new FlowLayout());
  JLabel label2 = new JLabel("Accepting State?");
  label2.setPreferredSize(new Dimension(100, 19));
  p2.add(label2);
  p2.add(jchkAccept);
  JPanel p3 = new JPanel();
  p3.setLayout(new FlowLayout()); 
  p2.setLayout(new FlowLayout());
  JLabel label3 = new JLabel("Starting State?");
  
  //label3.setPreferredSize(new Dimension(100, 19));
  p3.add(label3);
  p3.add(jchkStart = new JCheckBox("",(workSpacePanel.getFa().getStateIndex(state) == 0)? true : false));
  JPanel pMain = new JPanel();
  pMain.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 5));
  pMain.add(p1);
  pMain.add(p2);
  if (workSpacePanel.getFa().getStateIndex(state) != 0) 
  {
   pMain.add(p3);
  }
  pMain.setBorder(new TitledBorder("Enter Input"));
  JPanel pButtons = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
  pButtons.add(jbtOk = new JButton(" OK "));
  pButtons.add(jbtCancel = new JButton("Cancel"));
  getContentPane().setLayout(new BorderLayout()); 
  getContentPane().add(pMain, BorderLayout.CENTER);
  getContentPane().add(pButtons, BorderLayout.SOUTH);
  jtfNewStateName.addActionListener(this);
  jchkStart.addActionListener(this);
  jchkAccept.addActionListener(this);
  jbtOk.addActionListener(this);
  jbtCancel.addActionListener(this);  
 }

 //~ Methods ***********************************************************
 /**
 * On an OK, the state name is checked if it is valid. If it is, the
 * state's parameters are edited. If the starting state is checked,
 * then the state is moved to the top of the state list.
 *
 * @param e the action event generated by the dialog box
 */
 public void actionPerformed(ActionEvent e) 
 {
  if (e.getSource() instanceof JButton) 
  {
   if (e.getSource() == jbtOk)  
   {
    this.setVisible(false);
    this.state = workSpacePanel.getSelectedState();
    this.name = jtfNewStateName.getText();
    this.accepting = jchkAccept.isSelected();
    if ((FAChecker.isValidStateName(this, name)) && (FAChecker.isStateNameAvailable(this,workSpacePanel.getFa(),name, state))) 
    {
     this.state.setName(name);
     if (jchkStart.isSelected()) 
     {
      workSpacePanel.getFa().setStartingState(this.state);
     }
     this.state.setAcceptFlag(this.accepting);
    }
    workSpacePanel.repaint();
   } 
   else if (e.getSource() == jbtCancel) 
   {
    this.setVisible(false);
    workSpacePanel.repaint();
   }
  }
  if (e.getSource() instanceof JCheckBox) 
  {
   if (jchkAccept.isSelected()) 
   {
    this.accepting = false;
   } 
   else 
   {
    this.accepting = true;
   }
  }
  if (e.getSource() instanceof JCheckBox) 
  {
   if (jchkStart.isSelected()) 
   {
    this.start = true;
   } 
   else 
   {
    this.start = false; 
   }
  }  
 }
}
