 

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
* Implements a dialog box for editing a transition.
*/
public class EditTransitionDialogBox extends JDialog implements ActionListener 
{
 //~ Instance variables ************************************************
 /** Cancel button. */
 private JButton jbtCancel;

 /** OK button. */
 private JButton jbtOK;
 
 /** Textfield for transition input. */
 private JTextField jtfInput;

 /** The transition being edited. */ 
 private Transition transition;

 /** The current workspace panel. */
 private WorkSpacePanel workSpacePanel;

 /** The transition input. */
 private char TransInput;

 //~ Constructors ******************************************************
 /**
 * Generates the edit transition dialog box.
 *
 * @param parent the parent component for this dialog box
 * @param inputSpacePanel the workspace panel of the FA
 * @param t the transition to be edited
 */
 public EditTransitionDialogBox(JFrame parent,WorkSpacePanel inputSpacePanel,Transition t) 
 {
  workSpacePanel = inputSpacePanel;
  setTitle("Edit Transition");
  setSize(230, 130);
  JPanel p1 = new JPanel();
  p1.setLayout(new FlowLayout());
  p1.add(new JLabel("Input Symbol:"));
  transition = t;
  TransInput = t.getInput();
  String input = String.valueOf(TransInput);
  jtfInput = new JTextField(input, 10);
  p1.add(jtfInput);
  JPanel p2 = new JPanel();
  p2.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 2));
  p2.setBorder(new TitledBorder("Enter Input Symbol for Transition"));
  p2.add(p1);
  JPanel pButtons = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
  pButtons.add(jbtOK = new JButton(" OK "));
  pButtons.add(jbtCancel = new JButton("Cancel"));
  getContentPane().setLayout(new BorderLayout());
  getContentPane().add(p2, BorderLayout.CENTER);
  getContentPane().add(pButtons, BorderLayout.SOUTH);
  jbtOK.addActionListener(this);
  jbtCancel.addActionListener(this);
 }

 //~ Methods ***********************************************************
 /**
 * Generates the next action. On an OK, the transition input is
 * checked for validity, and if it valid the transition object itself
 * is edited.
 *
 * @param e the action event generated by the dialog box
 */
 public void actionPerformed(ActionEvent e) 
 {
  if (e.getSource() instanceof JButton) 
  {
   if (e.getSource() == jbtOK) 
   {
    FA fa = workSpacePanel.getFa();
    State source = transition.getSource();
    if (FAChecker.isValidInput(this, fa, jtfInput.getText()) && ((jtfInput.getText().charAt(0) == transition.getInput()) || FAChecker.isValidTransition(this, fa, source,jtfInput.getText().charAt(0)))) 
    {
     this.setVisible(false);
     transition.setInput(jtfInput.getText().charAt(0));
    }
   } 
   else if (e.getSource() == jbtCancel) 
   {
    this.setVisible(false);
   }
   workSpacePanel.repaint();
  }
 }
}
