/**
* Title: Graphical FA Editor Description: A graphical editor and
* simulator for deterministice finite automata (FA). Copyright:
* Copyright (c) 2004
*/

 
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

/**
* Implements the main graphical user interface (GUI) for the Graphical
* FA Editor.
*/

public class FAEditor extends JFrame 
{

//~ Static variables/initializers *************************************
/** The workspace panel component of the FA editor. */

 
 
 private static WorkSpacePanel workSpacePanel = new WorkSpacePanel();

 //~ Instance variables ************************************************
/** Instance variable for the working FA. */
 private FA fa = new FA();

/**********************************************************************
* The current working directory. Set to null on initialization, and
* set only on successful opening or importing of files.
**********************************************************************/
 private File currentDir = null;

/**
* The current working file. Set to null on initialization, and set
* only on successful opening or importing of files.
*/
 private File currentFile = null;

/** Toolbar button for adding a new state. */
 private JButton buttonAddState;

/** Toolbar button for adding a new transition. */
 private JButton buttonAddTrans;

/** Toolbar button for deleting a state. */
 private JButton buttonDeleteState;

/** Toolbar button for deleting a transition. */
 private JButton buttonDeleteTrans;

/** Toolbar button for editing the input alphabet. */
 private JButton buttonEditIA;

/** Toolbar button for editing a state. */
 private JButton buttonEditState;

/** Toolbar button for editing a transition. */
 private JButton buttonEditTrans;

/** Toolbar button for exporting a FA to file. */
 private JButton buttonExportFA;

/** Toolbar button for importing a FA from file. */
 private JButton buttonImportFA;

/** Toolbar button for creating a new FA. */
 private JButton buttonNewFA;

/** Toolbar button for opening a FA from file. */
 private JButton buttonOpenFA;

/** Toolbar button for renaming the FA. */
 private JButton buttonRenameFA;

/** Toolbar button for saving the FA to file. */
 private JButton buttonSaveFA;

/** Toolbar button for saving the FA to another file. */
 private JButton buttonSaveFAAs;

/** Toolbar button for simulating a FA with an input string. */
 private JButton buttonSimulate;
 
/** Toolbar button for closing the current FA */
 private JButton buttonCloseEditor; 
 
/** Toolbar button for exiting, prompts the user to save before exiting */ 
 private JButton buttonExitEditor;
  
/** Toolbar button for generalizing the current FA **/
 private JButton buttonGeneralize;

/** The default file chooser dialog box. */
 private JFileChooser fchooser;

/** The file menu. */
 private JMenu menuFile;
 
 private JMenu menuState;
 
 private JMenu menuTransition;
 
 private JMenu menuEditFA;

/** The help menu. */
 private JMenu menuHelp;

/** The menu bar. */
 private JMenuBar menu;

 private JMenuItem menuItemAddState;
 
 private JMenuItem menuItemEditState;

 private JMenuItem menuItemDeleteState;
 
 private JMenuItem menuItemAddTransition;
 
 private JMenuItem menuItemEditTransition;
 
 private JMenuItem menuItemDeleteTransition;
 
 private JMenuItem menuItemSimulate;
 
 private JMenuItem menuItemGeneralize;
 
 private JMenuItem menuItemEditIA;
 
 private JMenuItem menuItemRenameFA;
 
/** File menu item for closing the current FA. */
 private JMenuItem menuItemFileClose;

/** File menu item for exiting the FA Editor. */
 private JMenuItem menuItemFileExit;

/** File menu item for exporting the current FA. */
 private JMenuItem menuItemFileExport;

/** File menu item for importing an FA text file. */
 private JMenuItem menuItemFileImport;

/** File menu item for creating a new FA. */
 private JMenuItem menuItemFileNew;

/** File menu item for opening an existing FA. */
 private JMenuItem menuItemFileOpen;

/** File menu item for saving the current FA. */
 private JMenuItem menuItemFileSave;

/** File menu item for saving a copy of the current FA. */ 
 private JMenuItem menuItemFileSaveAs;

/** Help menu item for the about box. */
 private JMenuItem menuItemHelpAbout;

/** File menu item for help contents. */
 private JMenuItem menuItemHelpContents;

/** The container for the toolbar. */
 private JPanel toolPanel;

/** The container for the workspace panel, used for scrollability. */
 private JScrollPane scrollPane;

/** Menu list separator. */
 private JSeparator menuSeparator1;

/** Menu list separator. */
 private JSeparator menuSeparator2;

/** The status bar, where status text may be displayed. */
 private JTextField statusBar;

/** The editor toolbar. */
 private JToolBar toolBar;

/** Toolbar separator. */
 private JToolBar.Separator toolbarSeparator1;

/** Toolbar separator. */
 private JToolBar.Separator toolbarSeparator2;

/** Toolbar separator. */
 private JToolBar.Separator toolbarSeparator3;

/** String arguments for various method calls. */
 private String[] args = new String[2];

//~ Constructors ******************************************************
/**
* Default constructor for the FAEditor class. This method
* initializes the components of the editor's GUI and the FA editing
* workspace.
*/
 public FAEditor() 
 {
  scrollPane = new JScrollPane(workSpacePanel,
               JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
               JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
  statusBar = new JTextField("Graphical FA Editor");
  toolPanel = new JPanel();
  toolBar = new JToolBar();
  buttonNewFA = new JButton(new ImageIcon("icons/newfile.gif"));
  buttonOpenFA = new JButton(new ImageIcon("icons/openfile.gif"));
  buttonSaveFA = new JButton(new ImageIcon("icons/savefile.gif"));
  buttonSaveFAAs = new JButton(new ImageIcon("icons/savefileas.gif"));
  buttonImportFA = new JButton(new ImageIcon("icons/import.gif"));
  buttonExportFA = new JButton(new ImageIcon("icons/export.gif"));
  buttonAddState = new JButton(new ImageIcon("icons/addstate.gif"));
  buttonEditState = new JButton(new ImageIcon("icons/editstate.gif"));
  buttonDeleteState = new JButton(new ImageIcon("icons/deletestate.gif"));
  buttonAddTrans = new JButton(new ImageIcon("icons/addtransition.gif"));
  buttonEditTrans = new JButton(new ImageIcon("icons/edittransition.gif"));
  buttonDeleteTrans = new JButton(new ImageIcon("icons/deletetransition.gif"));
  buttonEditIA = new JButton(new ImageIcon("icons/editalphabet.gif"));
  buttonRenameFA = new JButton(new ImageIcon("icons/renamefa.gif"));
  buttonSimulate = new JButton(new ImageIcon("icons/simulate.gif"));
  buttonCloseEditor = new JButton(new ImageIcon("icons/closefa.gif"));
  buttonExitEditor = new JButton(new ImageIcon("icons/exitfa.gif"));
  buttonGeneralize = new JButton(new ImageIcon("icons/gen.gif")); 
  toolbarSeparator1 = new JToolBar.Separator();
  toolbarSeparator2 = new JToolBar.Separator();
  toolbarSeparator3 = new JToolBar.Separator();
  menu = new JMenuBar();
  menuFile = new JMenu("File");
  menuItemFileNew = new JMenuItem("New File", 'N');
  menuItemFileOpen = new JMenuItem("Open File", 'O');
  menuItemFileSave = new JMenuItem("Save File", 'S');
  menuItemFileSaveAs = new JMenuItem("Save File As...", 'A');
  menuItemFileClose = new JMenuItem("Close File", 'C');
  menuItemFileImport = new JMenuItem("Import FA", 'I');
  menuItemFileExport = new JMenuItem("Export FA", 'E');
  menuItemFileExit = new JMenuItem("Exit", 'X');
  menuHelp = new JMenu("Help");
  menuItemHelpContents = new JMenuItem("Contents", 'C');
  menuItemHelpAbout = new JMenuItem("About", 'A');
  menuSeparator1 = new JSeparator();
  menuSeparator2 = new JSeparator();

  menuState = new JMenu("State");
  menuItemAddState = new JMenuItem("Add State", 'A');
  menuItemEditState = new JMenuItem("Edit State", 'E');
  menuItemDeleteState = new JMenuItem("Delete State", 'D');
  
  menuTransition = new JMenu("Transition");
  menuItemAddTransition = new JMenuItem("Add Transition", 'A');
  menuItemEditTransition = new JMenuItem("Edit Transition", 'E');
  menuItemDeleteTransition = new JMenuItem("Delete Transition", 'D');
  
  menuEditFA = new JMenu("Edit FA");
  menuItemGeneralize = new JMenuItem("Generalize", 'A');
  menuItemEditIA = new JMenuItem("Edit Input Alphabet", 'E');
  menuItemRenameFA = new JMenuItem("Rename FA", 'D');
  menuItemSimulate = new JMenuItem("Simulate", 'D');
/* Create the toolbar and the toolbar buttons. */

  toolPanel.setLayout(new BorderLayout());
  toolBar.setFloatable(false);

  buttonNewFA.setMargin(new Insets(0, 0, 0, 0));
  buttonNewFA.setToolTipText("Create New FA");
  buttonNewFA.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    newFAActionPerformed();
   }
  });
  toolBar.add(buttonNewFA);

  buttonOpenFA.setMargin(new Insets(0, 0, 0, 0));
  buttonOpenFA.setToolTipText("Open FA File");
  buttonOpenFA.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    openFAActionPerformed();
   }
  });
  toolBar.add(buttonOpenFA);

  buttonSaveFA.setMargin(new Insets(0, 0, 0, 0));
  buttonSaveFA.setToolTipText("Save FA File");
  buttonSaveFA.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt)  
   {
    saveFAActionPerformed(currentFile);
   }
  });
  toolBar.add(buttonSaveFA);

  buttonSaveFAAs.setMargin(new Insets(0, 0, 0, 0));
  buttonSaveFAAs.setToolTipText("Save FA File As");
  buttonSaveFAAs.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt)  
   {
    saveFAAsActionPerformed(null);
   }
  });
  toolBar.add(buttonSaveFAAs);

  buttonCloseEditor.setMargin(new Insets(0, 0, 0, 0));
  buttonCloseEditor.setToolTipText("Close FA Editor");
  buttonCloseEditor.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt)  
   {
    closeFAActionPerformed();
   }
  });
  toolBar.add(buttonCloseEditor);

  buttonImportFA.setMargin(new Insets(0, 0, 0, 0));
  buttonImportFA.setToolTipText("Import FA File");
  buttonImportFA.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt)  
   {
    importFAActionPerformed();
   }
  });
  toolBar.add(buttonImportFA);

  buttonExportFA.setMargin(new Insets(0, 0, 0, 0));
  buttonExportFA.setToolTipText("Export to FA File");
  buttonExportFA.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    exportFAActionPerformed();
   }
  });
  toolBar.add(buttonExportFA);

  buttonExitEditor.setMargin(new Insets(0, 0, 0, 0));
  buttonExitEditor.setToolTipText("Exit FA Editor");
  buttonExitEditor.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt)  
   {
    exitActionPerformed();
   }
  });
  toolBar.add(buttonExitEditor);
  toolBar.add(toolbarSeparator1);
  buttonAddState.setMargin(new Insets(0, 0, 0, 0));
  buttonAddState.setToolTipText("Add State");
  buttonAddState.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    addStateActionPerformed();
   }
  });
  toolBar.add(buttonAddState);

  buttonEditState.setMargin(new Insets(0, 0, 0, 0));
  buttonEditState.setToolTipText("Edit State");
  buttonEditState.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    editStateActionPerformed();
   }
  });
  toolBar.add(buttonEditState);

  buttonDeleteState.setMargin(new Insets(0, 0, 0, 0));
  buttonDeleteState.setToolTipText("Delete State");
  buttonDeleteState.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    deleteStateActionPerformed();
   }
  });
  toolBar.add(buttonDeleteState);

  toolBar.add(toolbarSeparator2);

  buttonAddTrans.setMargin(new Insets(0, 0, 0, 0));
  buttonAddTrans.setToolTipText("Add Transition");
  buttonAddTrans.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    addTransitionActionPerformed();
   }
  });
  toolBar.add(buttonAddTrans);

  buttonEditTrans.setMargin(new Insets(0, 0, 0, 0));
  buttonEditTrans.setToolTipText("Edit Transition");
  buttonEditTrans.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    editTransitionActionPerformed(); 
   } 
  });
  toolBar.add(buttonEditTrans);

  buttonDeleteTrans.setMargin(new Insets(0, 0, 0, 0));
  buttonDeleteTrans.setToolTipText("Delete Transition");  
  buttonDeleteTrans.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    deleteTransitionActionPerformed();
   }
  });
  toolBar.add(buttonDeleteTrans);

  toolBar.add(toolbarSeparator3);

  buttonGeneralize.setToolTipText("Generalize FA Type");
  buttonGeneralize.setMargin(new Insets(0, 0, 0, 0));
  buttonGeneralize.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    generalizeActionPerformed();
   }
  });
  toolBar.add(buttonGeneralize);
  
  buttonEditIA.setToolTipText("Edit Input Alphabet");
  buttonEditIA.setMargin(new Insets(0, 0, 0, 0));
  buttonEditIA.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    editInputAlphabetActionPerformed();
   }
  });
  toolBar.add(buttonEditIA);

  buttonRenameFA.setToolTipText("Rename FA");
  buttonRenameFA.setMargin(new Insets(0, 0, 0, 0));
  buttonRenameFA.addActionListener(new ActionListener() 
  {  
   public void actionPerformed(ActionEvent evt) 
   {
    renameFAActionPerformed();
   }
  });
  toolBar.add(buttonRenameFA);

  buttonSimulate.setToolTipText("Simulate FA");
  buttonSimulate.setMargin(new Insets(0, 0, 0, 0));
  buttonSimulate.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    simulateActionPerformed();
   }
  });
  toolBar.add(buttonSimulate);

  toolPanel.add(toolBar, BorderLayout.CENTER);

/* Create the menu bar and the menu items */

  menuFile.setText("File");
  
  menuItemFileNew.setIcon(new ImageIcon("icons/newfile.gif"));
  menuItemFileNew.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    newFAActionPerformed();
   }
  });
  menuFile.add(menuItemFileNew);

  menuItemFileOpen.setIcon(new ImageIcon("icons/openfile.gif"));
  menuItemFileOpen.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    openFAActionPerformed();
   }
  });
  menuFile.add(menuItemFileOpen);

  menuItemFileSave.setIcon(new ImageIcon("icons/savefile.gif"));
  menuItemFileSave.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    saveFAActionPerformed(currentFile);
   }
  });
  menuFile.add(menuItemFileSave);

  menuItemFileSaveAs.setIcon(new ImageIcon("icons/savefileas.gif"));
  menuItemFileSaveAs.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    saveFAActionPerformed(null);
   }
  });
  menuFile.add(menuItemFileSaveAs);

  menuItemFileClose.setIcon(new ImageIcon("icons/closefa.gif"));
  menuItemFileClose.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    closeFAActionPerformed();
   }
  });
  menuFile.add(menuItemFileClose);

  menuFile.add(menuSeparator1);
 
  menuItemFileImport.setIcon(new ImageIcon("icons/import.gif"));
  menuItemFileImport.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    importFAActionPerformed();
   }
  });
  menuFile.add(menuItemFileImport);

  menuItemFileExport.setIcon(new ImageIcon("icons/export.gif"));
  menuItemFileExport.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    exportFAActionPerformed();
   }
  });
  menuFile.add(menuItemFileExport);

  menuFile.add(menuSeparator2);

  menuItemFileExit.setIcon(new ImageIcon("icons/exitfa.gif"));
  menuItemFileExit.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    exitActionPerformed();
   }
  });
  menuFile.add(menuItemFileExit);
  menu.add(menuFile);
  
  
  menuState.setText("State");
  menuState.setToolTipText("State Functions");
  
  menuItemAddState.setIcon(new ImageIcon("icons/addstate.gif"));
  menuItemAddState.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    addStateActionPerformed();
   }
  });
  menuState.add(menuItemAddState);

  menuItemEditState.setIcon(new ImageIcon("icons/editstate.gif"));
  menuItemEditState.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    editStateActionPerformed();
   }
  });
  menuState.add(menuItemEditState);

  menuItemDeleteState.setIcon(new ImageIcon("icons/deletetransition.gif"));
  menuItemDeleteState.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    deleteStateActionPerformed();
   }
  });
  menuState.add(menuItemDeleteState);
  menu.add(menuState);
  
  menuTransition.setText("Transition");
  menuTransition.setToolTipText("Transition Functions");
  
  menuItemAddTransition.setIcon(new ImageIcon("icons/addtransition.gif"));
  menuItemAddTransition.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    addTransitionActionPerformed();
   }
  });
  menuTransition.add(menuItemAddTransition);

  menuItemEditTransition.setIcon(new ImageIcon("icons/edittransition.gif"));
  menuItemEditTransition.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    editTransitionActionPerformed();
   }
  });
  menuTransition.add(menuItemEditTransition);

  menuItemDeleteTransition.setIcon(new ImageIcon("icons/deletetransition.gif"));
  menuItemDeleteTransition.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    deleteTransitionActionPerformed();
   }
  });
  menuTransition.add(menuItemDeleteTransition);
  menu.add(menuTransition);
  
  menuEditFA.setText("Edit FA");
  menuEditFA.setToolTipText("Edit the current FA");
  
  menuItemGeneralize.setIcon(new ImageIcon("icons/gen.gif"));
  menuItemGeneralize.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    generalizeActionPerformed();
   }
  });
  menuEditFA.add(menuItemGeneralize);

  menuItemEditIA.setIcon(new ImageIcon("icons/editalphabet.gif"));
  menuItemEditIA.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    editInputAlphabetActionPerformed();
   }
  });
  menuEditFA.add(menuItemEditIA);

  menuItemRenameFA.setIcon(new ImageIcon("icons/renamefa.gif"));
  menuItemRenameFA.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    renameFAActionPerformed();
   }
  });
  menuEditFA.add(menuItemRenameFA);
  
  menuItemSimulate.setIcon(new ImageIcon("icons/simulate.gif"));
  menuItemSimulate.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    simulateActionPerformed();
   }
  });
  menuEditFA.add(menuItemSimulate);
  menu.add(menuEditFA);
  
  //menuItemHelpContents.setIcon(new ImageIcon("icons/help.gif"));
  //menuHelp.add(menuItemHelpContents);
  menuItemHelpAbout.setIcon(new ImageIcon("icons/about.gif"));
  menuItemHelpAbout.addActionListener(new ActionListener() 
  {
   public void actionPerformed(ActionEvent evt) 
   {
    aboutActionPerformed(); 
   }
  });
  menuHelp.add(menuItemHelpAbout);

  menuFile.setMnemonic('F');
  menuHelp.setMnemonic('H');
  menu.add(menuHelp);

/* Add the components to the frame and set the frame properties. */

  Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
  setBounds((screenSize.width-800)/2,(screenSize.height-600)/2,800,600);
  setExtendedState(JFrame.MAXIMIZED_BOTH);
  workSpacePanel.setPreferredSize(workSpacePanel.getMaximumSize());
  getContentPane().add(scrollPane, BorderLayout.CENTER);  
  getContentPane().add(toolPanel, BorderLayout.NORTH);
  statusBar.setEditable(false);
  getContentPane().add(statusBar, BorderLayout.SOUTH);
  setJMenuBar(menu);
  setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
  setFont(new Font("Arial", 0, 10));
  setForeground(Color.black);
  setTitle("Graphical Finite Automata Editor");
 }

//~ Methods ***********************************************************
/**
* Calculates the position for the dialog box d, which should be at
* the center.
*
* @param d The dialog box to be centered.
*
* @return An array of size 2 containing the position of the center.
*/

 public int[] getCenter(JDialog d) 
 {
  int[] c = new int[2];
  Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
  int screenWidth = screenSize.width;
  int screenHeight = screenSize.height;
  Dimension size = d.getSize();
  c[0] = (screenWidth - size.width) / 2;
  c[1] = (screenHeight - size.height) / 2;
  return c;
 }

/**
* Sets the text to be displayed on the status bar.
*
* @param text The text to be displayed in the status bar.
*/

 public void setStatus(String text) 
 {
  if ((text == null) || (text.length() == 0)) 
  {
   text = "Graphical FA Editor";
  }
  this.statusBar.setText(text);
 }

/**
* Opens a message box about the FAEditor application.
*/

 public void aboutActionPerformed() 
 {
  FAChecker.displayInfo(this, 0);
 }

/**
* Opens a dialog box for specifying the parameters of a new state.
*/

 public void addStateActionPerformed() 
 {
  int[] c;
  workSpacePanel.resetTransSelected();
  workSpacePanel.resetStateSelected();
  workSpacePanel.repaint();
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true)) 
  {
   AddStateDialogBox addBox = new AddStateDialogBox(this, workSpacePanel);
   c = getCenter(addBox);
   addBox.setLocation(c[0], c[1]);
   addBox.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
   addBox.setVisible(true);
  }
 }

/**
* Opens a dialog box for specifying the parameter (input) of a new
* transition.
*/

 public void addTransitionActionPerformed() 
 {
  int[] c;
  workSpacePanel.resetTransSelected();
  workSpacePanel.resetStateSelected();
  workSpacePanel.repaint();
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true)) 
  {
   AddTransitionDialogBox addBox = new AddTransitionDialogBox(this, workSpacePanel);
   c = getCenter(addBox);
   addBox.setModal(true);
   addBox.setLocation(c[0], c[1]);
   addBox.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
   addBox.setVisible(true);
  }
 }

/**
* Closes the current FA file if there is any. TODO: Display a
* warning box for saving the FA before closing it.
*/

 public void closeFAActionPerformed() 
 {
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true)) 
  {
   int result = JOptionPane.showConfirmDialog(this,
                FAChecker.getWarningMsg(108,FAChecker.NO_ARGS),"Warning 108",
                JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
   if (result == JOptionPane.YES_OPTION) 
   {
    saveFAActionPerformed(currentFile);
    workSpacePanel.reset();
   } 
   else if (result == JOptionPane.CANCEL_OPTION) 
   {
    return;
   } 
   else if (result == JOptionPane.NO_OPTION) 
   {
    workSpacePanel.reset();
   }
  } 
  else 
  {
   workSpacePanel.reset();
  }
 }

/**
* Opens a confirmation dialog box for deleting the current selected
* state. If the confirmation dialog box returns an 'OK', the
* selected state is deleted from the FA, and the workspace is
* redrawn. If no state is selected, the method displays an error
* message.
*/

 public void deleteStateActionPerformed() 
 {
  if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))
  && (FAChecker.isStateSelected(this, workSpacePanel))) 
  {
   if (workSpacePanel.getFa().getStateIndex(workSpacePanel.getSelectedState()) == 0) 
   {
    FAChecker.displayError(this, 404);
   } 
   else if (FAChecker.displayWarning(this, 103) == FAChecker.OK) 
   {
    workSpacePanel.getFa().deleteState(workSpacePanel.getSelectedState());
    workSpacePanel.resetStateSelected();
   }
  }
  workSpacePanel.repaint();
 }

/**
* Opens a confirmation dialog box for deleting the current selected
* transition. If the confirmation dialog box returns an 'OK', the
* selected transition is deleted from the FA, and the workspace is
* redrawn. If no state is selected, the method displays an error
* message.
*/

 public void deleteTransitionActionPerformed() 
 {
  if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))
  && (FAChecker.isTransitionSelected(this, workSpacePanel))) 
  {
   if (FAChecker.displayWarning(this, 104) == FAChecker.OK) 
   {
    workSpacePanel.getFa().deleteTransition(workSpacePanel.getSelectedTransition());
    workSpacePanel.resetTransSelected();
   }
  }
  workSpacePanel.repaint();
 }

/**
 *Opens a Message Dialog prompting the user that the type will
 *be changed. A yes no option is taken
 *
 */
 public void generalizeActionPerformed()
 {
  int reply = 0;
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true)) 
  {
   char[] IA_old = workSpacePanel.getFa().getAlphabet();
   char[] IA_new = new char[IA_old.length + 1];
   if(WorkSpacePanel.isItFA())
   {
    JOptionPane.showMessageDialog(this,"The type of finite automaton\nis already a general one","Generalize",JOptionPane.INFORMATION_MESSAGE);
   }
   else
   {
    reply = JOptionPane.showConfirmDialog(this,"The finite automaton\nwill be changed to a general type","Generalize",JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
    if(reply == 0)
    {
     workSpacePanel.setFaType("fa");
     for(int i = 0; i<IA_old.length; i++)
     {
      if(i==0)
      {
       IA_new[i] = '';
      }
      IA_new[i+1] = IA_old[i];
     }
     workSpacePanel.getFa().setAlphabet(IA_new);
     workSpacePanel.setAlphabet(IA_new);
    }
    else
    {
     JOptionPane.showMessageDialog(this,"No action taken","Generalize",JOptionPane.INFORMATION_MESSAGE);	
    }     
   }
  }
 }
 
 

/**
* Opens a dialog box for editing the current input alphabet of the
* working FA.
*/

 public void editInputAlphabetActionPerformed() 
 {
  int[] c;
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true)) 
  {
   EditInputAlphabetDialogBox editBox = new EditInputAlphabetDialogBox(this, workSpacePanel);
   c = getCenter(editBox);
   editBox.setModal(true);
   editBox.setLocation(c[0], c[1]);
   editBox.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
   editBox.setVisible(true);
  }
 }

/**
* Opens a dialog box for editing the parameters of the current
* selected state. If no state is selected, the method displays an
* error message.
*/

 public void editStateActionPerformed() 
 {
  int[] c;
  if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))
  && (FAChecker.isStateSelected(this, workSpacePanel))) 
  {
   EditStateDialogBox editBox = new EditStateDialogBox(this, workSpacePanel,
   workSpacePanel.getSelectedState());
   c = getCenter(editBox);
   editBox.setModal(true);
   editBox.setLocation(c[0], c[1]);
   editBox.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
   editBox.setVisible(true);
  }
 }

/**
* Opens a dialog box for editing the parameters of the current
* selected transition. If no transition is selected, the method
* displays an error message.
*/

 public void editTransitionActionPerformed() 
 {
  int[] c;
  if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))
  && (FAChecker.isTransitionSelected(this, workSpacePanel))) 
  {
   EditTransitionDialogBox editBox = new EditTransitionDialogBox(this, workSpacePanel,
   workSpacePanel.getSelectedTransition());
   c = getCenter(editBox);
   editBox.setModal(true);
   editBox.setLocation(c[0], c[1]);
   editBox.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
   editBox.setVisible(true);
  }
 }

/**
* Called when the 'Exit' menu item in the menu bar is clicked. The
* application is halted with this method.
*/

 public void exitActionPerformed() 
 {
  if ((workSpacePanel.getFa() != null)) 
  {
   int result = JOptionPane.showConfirmDialog(this,
                FAChecker.getWarningMsg(107,FAChecker.NO_ARGS),"Warning 107",
                JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE);
   if (result == JOptionPane.YES_OPTION) 
   {
    saveFAActionPerformed(currentFile);
    System.exit(0);
   } 
   else if (result == JOptionPane.CANCEL_OPTION) 
   {
    return;
   } 
   else if (result == JOptionPane.NO_OPTION) 
   {
    System.exit(0);
   }
  } 
  else 
  {
   System.exit(0);
  }
 }

/**
* Exports the working FA to a text file with an FASPEC format. This
* method initially checks the current FA in the workspace for
* validitity and completeness using the methods in the FAChecker
* class. If the working FA is valid, it is formatted into FA
* specifications, and then saved to a file selected/created through
* a filechooser dialog box.
*/
 
 public void exportFAActionPerformed() 
 {
  File file;
  int response = 0;
  int reply = 0;
  
 if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))) 
  {
  if(WorkSpacePanel.isItDFA())
  {
   if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true)) && (FAChecker.isFAComplete(this, workSpacePanel.getFa()))) 
   {
   if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
    {
     reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
    }
    if(reply == 0)
    {
     fchooser = new JFileChooser(currentDir);
     fchooser.setDialogTitle("Export DFA to DFA File");
     if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) 
     {
      file = fchooser.getSelectedFile();
      args[0] = new String(file.getAbsolutePath());
      if (FAChecker.ExportFA(file, workSpacePanel.getFa())) 
      {
       FAChecker.displayInfo(this, 2, args);
       this.currentDir = file.getParentFile();
      } 
      else 
      {
       FAChecker.displayError(this, 201, args);
      }
     }	
    }    
   }
  }
  else if(WorkSpacePanel.isItNFA())
  {
   if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))) 
   {
   	if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
    {
     reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
    }
    if(reply == 0)
    {
     fchooser = new JFileChooser(currentDir);
     fchooser.setDialogTitle("Export NFA to NFA File");
     if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) 
     {
      file = fchooser.getSelectedFile();
      args[0] = new String(file.getAbsolutePath());
      if (FAChecker.ExportFA(file, workSpacePanel.getFa())) 
      {
       FAChecker.displayInfo(this, 2, args);
       this.currentDir = file.getParentFile();
      } 
      else 
      {
       FAChecker.displayError(this, 201, args);
      }
     }	
    }    
   }
  }
   //FAChecker.displayError(this, 502, args); 
  else if(WorkSpacePanel.isIteNFA())
  {
   if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))) 
   {
   	if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
    {
     reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
    }
    if(reply == 0)
    {
     fchooser = new JFileChooser(currentDir);
     fchooser.setDialogTitle("Export eNFA to an eNFA File");
     if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) 
     {
      file = fchooser.getSelectedFile();
      args[0] = new String(file.getAbsolutePath());
      if (FAChecker.ExportFA(file, workSpacePanel.getFa())) 
      {
       FAChecker.displayInfo(this, 2, args);
       this.currentDir = file.getParentFile();
      } 
      else 
      {
       FAChecker.displayError(this, 201, args);
      }
     }	
    }    
   }
  }
  else if(WorkSpacePanel.isItFA())
  {
   response = FAChecker.checkTypeOfFA(workSpacePanel.getFa());  
   if(response == 0)
   {   	
   	if(WorkSpacePanel.isItDFA())
    {   
     if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))) 
     {
      if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
      {
       reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
      }
      if(reply == 0)
      {
       fchooser = new JFileChooser(currentDir);
       fchooser.setDialogTitle("Export DFA to DFA File");
       if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) 
       {
        file = fchooser.getSelectedFile();
        args[0] = new String(file.getAbsolutePath());
        if (FAChecker.ExportFA(file, workSpacePanel.getFa())) 
        {
         FAChecker.displayInfo(this, 2, args);
         this.currentDir = file.getParentFile();
        } 
        else 
        {
         FAChecker.displayError(this, 201, args);
        }
       }      
      }
     }
    }
    else if(WorkSpacePanel.isItNFA())
    {
     if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))) 
     {
      if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
      {
       reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
      } 
      if(reply == 0)
      {
       fchooser = new JFileChooser(currentDir);
       fchooser.setDialogTitle("Export NFA to NFA File");
       if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) 
       {
        file = fchooser.getSelectedFile();
        args[0] = new String(file.getAbsolutePath());
        if (FAChecker.ExportFA(file, workSpacePanel.getFa())) 
        {
         FAChecker.displayInfo(this, 2, args);
         this.currentDir = file.getParentFile();
        } 
        else 
        {
         FAChecker.displayError(this, 201, args);
        }
       }
      }          
     }
    }
   
    else if(WorkSpacePanel.isIteNFA())
    {
     if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))) 
     {
      if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
      {
       reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
      } 
      if(reply == 0)
      {
       fchooser = new JFileChooser(currentDir);
       fchooser.setDialogTitle("Export eNFA to eNFA File");
       if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) 
       {
        file = fchooser.getSelectedFile();
        args[0] = new String(file.getAbsolutePath());
        if (FAChecker.ExportFA(file, workSpacePanel.getFa())) 
        {
         FAChecker.displayInfo(this, 2, args);
         this.currentDir = file.getParentFile();
        } 
        else 
        {
         FAChecker.displayError(this, 201, args);
        }
       }
      }          
     }
    }
   } 
  }
  }
  workSpacePanel.repaint();
 }

/**
* Opens a file chooser dialog box for selecting a text file. If the
* text file is a valid FA format file, the string content is checked
* using the FAParser class. This class also parses the string into
* the FA name, the input alphabet, the state list, and the
* transition list. These are used to create a new FA object, the
* new FA is passed to the workspace.
*/

 public void importFAActionPerformed() 
 {
  File file;
  int result = FAChecker.OK;
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), false)) 
  {
   result = FAChecker.displayWarning(this, 102);
  }
  if (result == FAChecker.OK) 
  {
   fchooser = new JFileChooser(currentDir);
   fchooser.setDialogTitle("Import an FA from File");
   if ((fchooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) && ((file = fchooser.getSelectedFile()).exists())) 
   {
    args[0] = file.getAbsolutePath();
    try 
    {
     LineNumberReader in = new LineNumberReader(new FileReader(file));
     FAParser parser = new FAParser(in, workSpacePanel);
     if (!parser.isError()) 
     {      
      
      fa = new FA(parser.getFAName());
      fa.setAlphabet(parser.getAlphabet());
      State[] stateList = parser.getStateList();
      Transition[] transitionList = parser.getTransitionList();
      for (int i = 0; i < stateList.length; i++) 
      { 
       State s = stateList[i];
       fa.addState(s);
      }
      for (int i = 0; i < transitionList.length; i++) 
      {
       Transition t = transitionList[i];
       fa.addTransition(t);
      }
      workSpacePanel.reset();
      workSpacePanel.setFA(fa);   
      WorkSpacePanel.setFaType(FAParser.getFAType());   
      this.currentDir = file.getParentFile();
      //setStatus("Importing FA file " + args[0]);
      JOptionPane.showMessageDialog(this,parser.getMessage());
     }  
     else if(parser.isError())
     {
      JOptionPane.showMessageDialog(this,
      FAChecker.getErrorMsg(103,FAChecker.NO_ARGS)+parser.getMessage(),
      "Error 103",JOptionPane.ERROR_MESSAGE); 
      setStatus("");
     }
    } catch (IOException e1) 
    {
    FAChecker.displayError(this, 101, args);
    this.currentFile = null;
    }
   }
  }
  workSpacePanel.repaint();    
 // }
 }

/**
* Main method for FAEditor class.
*
* @param args command line arguments
*/

 public static void main(String[] args) 
 {
  FAEditor frame = new FAEditor();
  workSpacePanel.setBackground(new Color(255, 255, 255));
  workSpacePanel.setContainer(frame);
  frame.setDefaultLookAndFeelDecorated(true);  
  frame.setVisible(true);  
 }
 
/**
* Opens a dialog box for creating a new FA.
*/

 public void newFAActionPerformed() 
 {
  int[] c;
  int result = FAChecker.OK;
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), false)) 
  {
   result = FAChecker.displayWarning(this, 101);
  }
  if (result == FAChecker.OK) 
  {
   this.currentFile = null;
   NewFileDialogBox frame = new NewFileDialogBox(this, workSpacePanel);
   c = getCenter(frame);
   frame.setLocation(c[0], c[1]);
   frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
   frame.setVisible(true);
  }
 }

 
/**
* Opens a filechooser dialog for selecting an existing FAEditor save
* file. The selected file must be saved as a binary object, and this
* method converts the binary contents into FA object. The FA
* object is then passed to the workspace.
*/

 public void openFAActionPerformed() 
 {
  File file;
  FileInputStream fis;
  ObjectInputStream in;
  String fatype = new String();
  int result = FAChecker.OK;

/* If there's a working FA, confirm to discard FA first. */

  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), false)) 
  {
   result = FAChecker.displayWarning(this, 102); 
  }
  if (result == FAChecker.OK) 
  {
   fchooser = new JFileChooser(currentDir);
   fchooser.setDialogTitle("Open an FA File");
   if ((fchooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
   && ((file = fchooser.getSelectedFile()).exists())) 
   {
    args[0] = file.getAbsolutePath();
    try 
    {
    fis = new FileInputStream(file);
    in = new ObjectInputStream(fis);
    
    fa = (FA) in.readObject(); 
    fatype = (String)in.readObject();  
    workSpacePanel.reset();    
    workSpacePanel.setFA(fa);
    workSpacePanel.setFaType(fatype);
    this.currentFile = file;
    this.currentDir = file.getParentFile();
    setStatus("Opened FA file " + args[0]);
    } catch (IOException e1) 
    {
     FAChecker.displayError(this, 101, args);
     this.currentFile = null;
    } catch (ClassNotFoundException e2) 
    {
     FAChecker.displayError(this, 101, args);
     this.currentFile = null;
    }
   }
  }
 workSpacePanel.repaint();
 }

/**
* Opens a dialog box for renaming the working FA.
*/

 public void renameFAActionPerformed() 
 {
  int[] c;
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true)) 
  {
   RenameFADialogBox renBox = new RenameFADialogBox(this, workSpacePanel);
   c = getCenter(renBox);
   renBox.setModal(true);
   renBox.setLocation(c[0], c[1]);
   renBox.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
   renBox.setVisible(true);
  }
 }

/**
* Saves the current FA in the workspace to a binary file.
*
* @param file The file into which the FA is saved. If it is null,
* this method opens a file chooser dialog box for selecting
* or creating a new save file.
*/

 public void saveFAActionPerformed(File file) 
 {
  FileOutputStream fos;
  ObjectOutputStream out;
  int response = 0;
  int reply = 0;
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true)) 
  {
   if(WorkSpacePanel.isItFA())
   {
    response = FAChecker.checkTypeOfFA(workSpacePanel.getFa());   
    if(response == 0)
    {
     if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
     {
      reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
     } 
     if(reply == 0)
     {
      if (file == null) 
      {
       fchooser = new JFileChooser(currentDir);
       fchooser.setDialogTitle("Save " + WorkSpacePanel.getFaType() + " File");
       if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
       {
        file = fchooser.getSelectedFile();
       }
      }
      if (file != null) 
      {
       args[0] = file.getAbsolutePath();
       try 
       {
        fos = new FileOutputStream(file);
        out = new ObjectOutputStream(fos);
        out.writeObject(workSpacePanel.getFa());
        out.writeObject(workSpacePanel.getFaType() );
        this.currentFile = file;
        this.currentDir = file.getParentFile();
        FAChecker.displayInfo(this, 1, args);
       } 
       catch (IOException e) 
       {
        FAChecker.displayError(this, 102, args);
       }
      }
     }     	
    }
   }
   else 
   {
   	if(WorkSpacePanel.isItDFA())
    {   
     if(FAChecker.isFAComplete(this, workSpacePanel.getFa()))
     {
      if (file == null) 
      {
       fchooser = new JFileChooser(currentDir);
       fchooser.setDialogTitle("Save " + WorkSpacePanel.getFaType() + " File");
       if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
       {
         file = fchooser.getSelectedFile();
       }
      }
      if (file != null) 
      {
       args[0] = file.getAbsolutePath();
       try 
       {
        fos = new FileOutputStream(file);
        out = new ObjectOutputStream(fos);
        out.writeObject(workSpacePanel.getFa());
        out.writeObject(workSpacePanel.getFaType() );
        this.currentFile = file;
        this.currentDir = file.getParentFile();
        FAChecker.displayInfo(this, 1, args);
       } 
       catch (IOException e) 
       {
       FAChecker.displayError(this, 102, args);
       }
      }      
     }         
    }
    else
    {
     if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
     {
      reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
     } 
     if(reply == 0)
     {
      if (file == null) 
      {
       fchooser = new JFileChooser(currentDir);
       fchooser.setDialogTitle("Save " + WorkSpacePanel.getFaType() + " File");
       if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
       {
        file = fchooser.getSelectedFile();
       }
      }
      if (file != null) 
      {
       args[0] = file.getAbsolutePath();
       try 
       {
        fos = new FileOutputStream(file);
        out = new ObjectOutputStream(fos);
        out.writeObject(workSpacePanel.getFa());
        out.writeObject(workSpacePanel.getFaType());
        this.currentFile = file;
        this.currentDir = file.getParentFile();
        FAChecker.displayInfo(this, 1, args);
       } 
       catch (IOException e) 
       {
        FAChecker.displayError(this, 102, args);
       }
      }
     }     
    }
   }   
  }
  workSpacePanel.repaint();
 }

//SAVE AS
 public void saveFAAsActionPerformed(File file) 
 {
  FileOutputStream fos;
  ObjectOutputStream out;
  int response = 0;
  int reply = 0;
  if (!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true)) 
  {
   if(WorkSpacePanel.isItFA())
   {
    response = FAChecker.checkTypeOfFA(workSpacePanel.getFa());   
    if(response == 0)
    {
     if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
     {
      reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
     } 
     if(reply == 0)
     {
      if (file == null) 
      {
       fchooser = new JFileChooser(currentDir);
       fchooser.setDialogTitle("Save FA File As");
       if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
       {
        file = fchooser.getSelectedFile();
       }
      }
      if (file != null) 
      {
       args[0] = file.getAbsolutePath();
       try 
       {
        fos = new FileOutputStream(file);
        out = new ObjectOutputStream(fos);
        out.writeObject(workSpacePanel.getFa());
        out.writeObject(workSpacePanel.getFaType());
        this.currentFile = file;
        this.currentDir = file.getParentFile();
        FAChecker.displayInfo(this, 1, args);
       } 
       catch (IOException e) 
       {
        FAChecker.displayError(this, 102, args);
       }
      }
     }     
    }
   }
   else 
   {
   	if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa()))
    {
     reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
    } 
    if(reply == 0)
    {
     if (file == null) 
     {
      fchooser = new JFileChooser(currentDir);
      fchooser.setDialogTitle("Save FA File As");
      if (fchooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
      {
       file = fchooser.getSelectedFile();
      }
     }
     if (file != null) 
     {
      args[0] = file.getAbsolutePath();
      try 
      {
      fos = new FileOutputStream(file);
      out = new ObjectOutputStream(fos);
      out.writeObject(workSpacePanel.getFa());
      out.writeObject(workSpacePanel.getFaType());
      this.currentFile = file;
      this.currentDir = file.getParentFile();
      FAChecker.displayInfo(this, 1, args);
      } 
      catch (IOException e) 
      {
       FAChecker.displayError(this, 102, args);
      }
     }
    }    
   }  
  }
  workSpacePanel.repaint();
 }
 

/**
* Opens the simulation dialog box to allow simulating the working FA
* with an input string. The FA is initially checked for validity
* and completeness using the methods in the FAChecker class. If it
* is not valid, an error message is displayed instead.
*/

 public void simulateActionPerformed() 
 {
  int[] c;
  int response = 0;
  int reply = 0;
  
  if(!FAChecker.isEmptyFA(this,workSpacePanel.getFa(),true) )
  {
   char[] tmpInputAlphabet = workSpacePanel.getFa().getAlphabet();  
   char[] myABC = new char[tmpInputAlphabet.length - 1];  
   if(WorkSpacePanel.isItFA())
   {
    response = FAChecker.checkTypeOfFA(workSpacePanel.getFa());  
    if(response == 0)
    {
     for (int i = 0; i < tmpInputAlphabet.length; i++)
     {
       if(i==0 && tmpInputAlphabet[0]=='')
      {
       for (int ctr = 0; ctr < tmpInputAlphabet.length-1; ctr++)
       {
         myABC[ctr]=tmpInputAlphabet[ctr+1];
       }
       fa.setAlphabet(myABC);
       workSpacePanel.setFaAlphabet(myABC);
      }
     }
      if(!FAChecker.doesAcceptingStateExist(workSpacePanel.getFa())) 
     {
      reply = JOptionPane.showConfirmDialog(null,"The "+WorkSpacePanel.getFaType()+" has no accepting state.\nAre you sure this is complete?","No Accepting State",JOptionPane.YES_NO_OPTION,JOptionPane.PLAIN_MESSAGE);
     }
     if(reply == 0)
     {
      if(WorkSpacePanel.isItDFA())
      {
       FASimulateDialogBox simBox = new FASimulateDialogBox(this, workSpacePanel);
       c = getCenter(simBox);
       simBox.setModal(true);
       simBox.setLocation(c[0], c[1]);
       simBox.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
       simBox.setVisible(true);	
      }     
      else 
      {     
       FAChecker.NotADFA(this); 
      }
     }    	     
    }   
   }
  }  
  else if(WorkSpacePanel.isItDFA())
  {
   if ((!FAChecker.isEmptyFA(this, workSpacePanel.getFa(), true))&&(FAChecker.isFAComplete(this, workSpacePanel.getFa())))
   { 	
    FASimulateDialogBox simBox = new FASimulateDialogBox(this, workSpacePanel);     
    c = getCenter(simBox);
    simBox.setModal(true);
    simBox.setLocation(c[0], c[1]);
    simBox.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    simBox.setVisible(true);        
   }
  }    
  else 
  {
   FAChecker.NotADFA(this);
  }      
 }
}
