 

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
/**
* Implements a dialog box for creating a new FA.
*/
public class NewFileDialogBox extends JDialog implements ActionListener 
{
//~ Instance variables 
/** the parent component of this dialog box */
private FAEditor container;
/** Cancel button. */
private JButton jbtCancel;
/** OK button. */
private JButton jbtOK;
/** Sets the accepting flag for the starting state. */

private JRadioButton DFARadioButton  = new JRadioButton("DFA");

private JRadioButton NFARadioButton  = new JRadioButton("NFA");

private JRadioButton eNFARadioButton  = new JRadioButton("eNFA");

private JRadioButton FARadioButton  = new JRadioButton("FA",true);

private JCheckBox jchkAccept;
/** Sets the FA name. */
private JTextField jtfFAName;
/** Sets the input alphabet string. */
private JTextField jtfInputAlphabet;
/** Sets the name of the starting state. */
private JTextField jtfStartingState;
/** The workspace for the FA. */
private WorkSpacePanel workSpacePanel;
//~ Constructors ******************************************************
/**
* The sole constructor for this dialog box class.
*
* @param parent the parent component of this dialog box
* @param inputSpacePanel the workspace for the FA
*/
	public NewFileDialogBox(JFrame parent, WorkSpacePanel inputSpacePanel) 
	{
		super(parent, true);
		workSpacePanel = inputSpacePanel;
		container = (FAEditor) parent;
		setTitle("Create a New FA");
		setSize(350, 270);
		
		ButtonGroup rbuttons = new ButtonGroup();		 
			rbuttons.add(FARadioButton);
			rbuttons.add(DFARadioButton);
			rbuttons.add(NFARadioButton);
			rbuttons.add(eNFARadioButton);
					
		JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel1.add(FARadioButton);
		JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));	
			panel2.add(DFARadioButton);
		JPanel panel3 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel3.add(NFARadioButton);
		JPanel panel4 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel4.add(eNFARadioButton);
		
		JPanel pMain1 = new JPanel(new FlowLayout(FlowLayout.LEFT));				
			pMain1.add(panel1);	
			pMain1.add(panel2);
			pMain1.add(panel3);
			pMain1.add(panel4);
			pMain1.setBorder(new TitledBorder("Choose type of FA"));
			
		JPanel p1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			p1.setLayout(new FlowLayout());
			JLabel label1 = new JLabel("FA Name");
			label1.setPreferredSize(new Dimension(90, 19));
			p1.add(label1);
			p1.add(jtfFAName = new JTextField("FA1", 20));
		
		JPanel p2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			p2.setLayout(new FlowLayout());
			JLabel label2 = new JLabel("Input Alphabet");
			label2.setPreferredSize(new Dimension(90, 19));
			p2.add(label2);			
			p2.add(jtfInputAlphabet = new JTextField("", 20));
		
		JPanel p3 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			JLabel label3 = new JLabel("Starting State");
			label3.setPreferredSize(new Dimension(90, 19));
			p3.add(label3);
			p3.add(jtfStartingState = new JTextField("q0", 8));
		
		JPanel p4 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			p4.add(new JLabel("Accepting State?"));
			p4.add(jchkAccept = new JCheckBox());
		
		JPanel pMain2 = new JPanel();
			pMain2.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 5));
			pMain2.add(p1);
			pMain2.add(p2);
			pMain2.add(p3);
			pMain2.add(p4);
			pMain2.setBorder(new TitledBorder("Enter New FA Info"));
		
		JPanel pButtons = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
			pButtons.add(jbtOK = new JButton(" OK "));
			pButtons.add(jbtCancel = new JButton("Cancel"));
			
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(pMain1, BorderLayout.NORTH);
		getContentPane().add(pMain2, BorderLayout.CENTER);
		getContentPane().add(pButtons, BorderLayout.SOUTH);
		jtfFAName.addActionListener(this);
		jtfInputAlphabet.addActionListener(this);
		jtfStartingState.addActionListener(this);
		jchkAccept.addActionListener(this);
		jbtOK.addActionListener(this);
		jbtCancel.addActionListener(this);
		DFARadioButton.addActionListener(this);
		NFARadioButton.addActionListener(this);
		eNFARadioButton.addActionListener(this);
		FARadioButton.addActionListener(this);
		setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
	}
//~ Methods ***********************************************************
/**
* Determines the next action for the FA. On an OK, the FA name,
* the input alphabet string, and the state name are checked for
* validity. If all are valid, they are then passed to the workspace
* panel along with a NEWFILE command. The state automatically
* becomes the starting state.
*
* @param e the action event generated by the dialog box
*/

	public void actionPerformed(ActionEvent e) 
	{
		if (DFARadioButton.isSelected()) 
		{			
			if (e.getSource() instanceof JButton) 
			{
				if (e.getSource() == jbtOK) 
				{
					workSpacePanel.setFaType("dfa");
					if (FAChecker.isValidFAName(this, jtfFAName.getText())&& FAChecker.isValidStateName(this,jtfStartingState.getText())&& FAChecker.isValidInputAlphabet(this,jtfInputAlphabet.getText())) 
					{
						workSpacePanel.reset();
						workSpacePanel.setFaType("dfa");
						workSpacePanel.setFaName(jtfFAName.getText());						
						workSpacePanel.setStateName(jtfStartingState.getText());
						workSpacePanel.setAlphabet(FAChecker.expandAlphabetString(jtfInputAlphabet.getText()));
						workSpacePanel.setCommand(WorkSpacePanel.NEWFILE);
						this.setVisible(false);						
					}
				} 
				else if (e.getSource() == jbtCancel) 
				{
					this.setVisible(false);
				}
				workSpacePanel.repaint();
			}
			if (e.getSource() instanceof JCheckBox) 
			{
				if (jchkAccept.isSelected()) 
				{
					workSpacePanel.setAccepting(true);
				} 
				else 
				{
					workSpacePanel.setAccepting(false);
				}
			}		
		} 
		else if (NFARadioButton.isSelected()) 
		{			
			if (e.getSource() instanceof JButton) 
			{
				if (e.getSource() == jbtOK) 
				{
					workSpacePanel.setFaType("nfa");
					if (FAChecker.isValidFAName(this, jtfFAName.getText())&& FAChecker.isValidStateName(this,jtfStartingState.getText())&& FAChecker.isValidInputAlphabet(this,jtfInputAlphabet.getText())) 
					{
						workSpacePanel.reset();
						workSpacePanel.setFaType("nfa");
						workSpacePanel.setFaName(jtfFAName.getText());
						workSpacePanel.setStateName(jtfStartingState.getText());
						workSpacePanel.setAlphabet(FAChecker.expandAlphabetString(jtfInputAlphabet.getText()));
						workSpacePanel.setCommand(WorkSpacePanel.NEWFILE);
						this.setVisible(false);						
					}
				} 
				else if (e.getSource() == jbtCancel) 
				{
					this.setVisible(false);
				}
				workSpacePanel.repaint();
			}
			if (e.getSource() instanceof JCheckBox) 
			{
				if (jchkAccept.isSelected()) 
				{
					workSpacePanel.setAccepting(true);
				} 
				else 
				{
					workSpacePanel.setAccepting(false);
				}
			}		
		} 
		else if (eNFARadioButton.isSelected()) 
		{			
			if (e.getSource() instanceof JButton) 
			{
				if (e.getSource() == jbtOK) 
				{					
					workSpacePanel.setFaType("enfa");
					if (FAChecker.isValidFAName(this, jtfFAName.getText())&& FAChecker.isValidStateName(this,jtfStartingState.getText())&& FAChecker.isValidInputAlphabet(this,jtfInputAlphabet.getText())) 
					{			
						workSpacePanel.reset();
						workSpacePanel.setFaType("enfa");			
						workSpacePanel.setFaName(jtfFAName.getText());
						workSpacePanel.setStateName(jtfStartingState.getText());
						workSpacePanel.setAlphabet(FAChecker.expandAlphabetString(jtfInputAlphabet.getText()));
						workSpacePanel.setCommand(WorkSpacePanel.NEWFILE);
						this.setVisible(false);	
						FAChecker.displayInfo(this, 3);			
					}
				} 
				else if (e.getSource() == jbtCancel) 
				{
					this.setVisible(false);
				}
				workSpacePanel.repaint();
			}
			if (e.getSource() instanceof JCheckBox) 
			{
				if (jchkAccept.isSelected()) 
				{
					workSpacePanel.setAccepting(true);
				} 
				else 
				{
					workSpacePanel.setAccepting(false);
				}
			}		
		} 
		else if (FARadioButton.isSelected()) 
		{			
			if (e.getSource() instanceof JButton) 
			{
				if (e.getSource() == jbtOK) 
				{				
					workSpacePanel.setFaType("fa");	
					if (FAChecker.isValidFAName(this, jtfFAName.getText())&& FAChecker.isValidStateName(this,jtfStartingState.getText())&& FAChecker.isValidInputAlphabet(this,jtfInputAlphabet.getText())) 
					{
						workSpacePanel.reset();	
						workSpacePanel.setFaType("fa");									
						workSpacePanel.setFaName(jtfFAName.getText());
						workSpacePanel.setStateName(jtfStartingState.getText());
						workSpacePanel.setAlphabet(FAChecker.expandAlphabetString(jtfInputAlphabet.getText()));
						workSpacePanel.setCommand(WorkSpacePanel.NEWFILE);
						this.setVisible(false);
						FAChecker.displayInfo(this, 3);
						
					}
				} 
				else if (e.getSource() == jbtCancel) 
				{
					this.setVisible(false);
				}
				workSpacePanel.repaint();
			}
			if (e.getSource() instanceof JCheckBox) 
			{
				if (jchkAccept.isSelected()) 
				{
					workSpacePanel.setAccepting(true);
				} 
				else 
				{
					workSpacePanel.setAccepting(false);
				}
			}		
		}	
	}
}
