 
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

/**
* Implements a dialog box for renaming the FA.
*/
public class RenameFADialogBox extends JDialog implements ActionListener 
{
 //~ Instance variables ************************************************
 /** Cancel button. */
 private JButton jbtCancel;

 /** OK button. */
 private JButton jbtOK;

 /** Editable text field for FA name. */ 
 private JTextField newFAName;

 /** Non-editable text field for FA name. */
 private JTextField oldFAName;

 /** The new FA name. */ 
 private String newFA;

 /** The unedited FA name. */
 private String oldFA;
 
 /** The workspace panel for the FA. */
 private WorkSpacePanel workSpacePanel;
 
 //~ Constructors ******************************************************
 /**
 * The sole constructor for this dialog box class.
 *
 * @param parent the parent component of this dialog box
 * @param inputSpacePanel the workspace panel for this FA
 */
 public RenameFADialogBox(JFrame parent,WorkSpacePanel inputSpacePanel) 
 {
  super(parent, true);
  workSpacePanel = inputSpacePanel;
  setTitle("Rename FA");
  setSize(300, 170);
  JPanel p1 = new JPanel();
  p1.setLayout(new FlowLayout());
  JLabel label1 = new JLabel("Old FA Name");
  label1.setPreferredSize(new Dimension(100, 19));
  p1.add(label1);
  p1.add(oldFAName = new JTextField(10));
  oldFAName.setEditable(false);
  oldFA = inputSpacePanel.getFa().getName();
  oldFAName.setText(oldFA);
  JPanel p2 = new JPanel();
  p2.setLayout(new FlowLayout());
  JLabel label2 = new JLabel("New FA Name");
  label2.setPreferredSize(new Dimension(100, 19));
  p2.add(label2);
  p2.add(newFAName = new JTextField(10));
  newFAName.setText(oldFA);
  JPanel pMain = new JPanel();
  pMain.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 5));
  pMain.add(p1); 
  pMain.add(p2);
  pMain.setBorder(new TitledBorder("Rename FA"));
  JPanel pButtons = new JPanel();
  pButtons.setLayout(new FlowLayout(FlowLayout.RIGHT, 0, 0));
  pButtons.add(jbtOK = new JButton("OK"));
  pButtons.add(jbtCancel = new JButton("Cancel"));
  getContentPane().setLayout(new BorderLayout());
  getContentPane().add(pMain, BorderLayout.CENTER);
  getContentPane().add(pButtons, BorderLayout.SOUTH);
  newFAName.addActionListener(this);
  jbtOK.addActionListener(this);
  jbtCancel.addActionListener(this);
 }
 
 //~ Methods ***********************************************************
 /**
 * Generates the next action. On an OK, the new FA name is checked 
 * for validity, and if it is valid, the FA object's name field is
 * edited.
 *
 * @param e the action event generated by the dialog box
 */
 public void actionPerformed(ActionEvent e) 
 {
  if (e.getSource() instanceof JButton) 
  {
   if (e.getSource() == jbtOK) 
   {
    String name = newFAName.getText();
    if (FAChecker.isValidFAName(this, name)) 
    {
     this.setVisible(false);
     workSpacePanel.getFa().setName(name);
    }
   } 
   else if (e.getSource() == jbtCancel) 
   {
    this.setVisible(false);
   }
  }
  workSpacePanel.repaint();
 }
}
