 

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
* Implements the workspace or editing environment for a FA. Graphical
* methods are defined within the workspace for FA transition diagrams.
* The WorkSpacePanel class is the backbone of the Graphical FA Editor.
*/

public class WorkSpacePanel extends JPanel implements MouseListener, MouseMotionListener 
{
 //~ Static variables/initializers 
 
 /** The default command. */
 public static final int NONE = 0;
 
 /** The command for creating a new file. */
 public static final int NEWFILE = 1;

 /** The command for adding a new state. */
 public static final int ADDSTATE = 2;

 /** The command for adding a new transition. */
 public static final int ADDTRANSITION = 3;

 //~ Instance variables 
 /** The working FA for the workspace. */
 private FA fa = null;
 
 /**
 * The default container for the workspace, preferably the FAEditor
 * frame.
 */
 private static FAEditor container;

 /** Initiates Java graphics capabilities for the workspace. */
 private Graphics g;
 
 /**
 * Transition instance variable for the destination state, set when
 * adding a new transition.
 */
 private State DstState = null;
 
 /**
 * Transition instance variable for the source state, set when adding
 * a new transition.
 */
 private State SrcState = null;
 
 /** The current selected state. */
 private State selectedState = null;
 
 /** An instance variable for a FA state. */
 private State state = null;
 
 /**
 * The FA name, set when creating a new FA, opening, or importing a
 * FA.
 */
 private String faName = "File_1";
 
 /**
 * The FA type, set when creating a new FA, opening, or importing a
 * FA.
 */
 private static String faType;
 
 
 /**
 * The state name, used when adding a new state or creating a new FA.
 */
 private String stateName = "q0";
 
 /** The current selected transition. */
 private Transition selectedTrans = null;
 
 /** An instance variable for a FA transition. */
 private Transition transition = null;
 
 /** The input alphabet. */
 private char[] IA;
 
 /** String arguments for use by various method calls. */
 private String[] args = new String[2];
 
 /** The accepting flag of a state. */
 private boolean accepting = false;
 
 /** A transition input. */
 private char input;
 
 /** The default radius for a circle representation of a state. */
 private int Radius = 25;
 
 /**
 * The current workspace command. This sets the action for the
 * workspace. The commands are defined as follows: NONE = default
 * command, always reset to this after an action; NEWFILE = command
 * for creating a new file; ADDSTATE = command for adding a new
 * state; ADDTRANSITION = command for adding a new transition.
 */
 private int command = 0;
 
 /**
 * Variable for the x-coordinate of the destination of a transition
 * line.
 */
 private int dstX;
 
 /**
 * Variable for the y-coordinate of the destination of a transition
 * line.
 */
 private int dstY;
 
 /** Variable for the x-coordinate of the source of a transition line. */
 private int srcX;
 
 /** Variable for the y-coordinate of the source of a transition line. */
 private int srcY;

//handles the color 
Color lightblue = new Color(141,198,254);
Color lightgreen = new Color(142,253,236);
Color darkblue = new Color(10,19,107);
Color lightpink = new Color(254,158,141);
Color lightyellow = new Color(248,254,169);
Color darkpink = new Color(254,134,120);
Color blue_accept = new Color(102,165,242);
 
 //~ Constructors 
 /**
 * Sole constructor for the workspace panel. This initiates the mouse
 * and mouse motion listeners which are important in editing the
 * current FA.
  */
 public WorkSpacePanel()
 {
  addMouseListener(this);
  addMouseMotionListener(this);
 }

 //~ Methods 
 /**
 * Sets the accepting flag of a state to the given value.
 *
 * @param accepting the accepting flag for the currently set state
 */
 public void setAccepting(boolean accepting) 
 {
  this.accepting = accepting;
 }

 /**
 * Sets the input alphabet of the fa to the given alphabet.
 *
 * @param input the input alphabet of the fa
 */
 public void setAlphabet(char[] input) 
 {
  this.IA = input;  
 }

 /**
 * Sets the command for the workspace panel to indicate the next
 * action.
 *
 * @param x the command for the workspace panel
 */
 public void setCommand(int x) 
 {
  this.command = x;
 }

 /**
 * Sets the parent component of the workspace panel (usually the
 * FAEditor frame)
 *
 * @param c parent component for the panel
 */
 public void setContainer(FAEditor c) 
 {
  this.container = c;
 }

 /**
 * Sets the working FA for the workspace panel.
 *
 * @param fa the working FA
 */
 public void setFA(FA fa) 
 {
  this.fa = fa;
 }

 public void setFaAlphabet(char[] input)
 {
 	fa.setAlphabet(input);
 }
 
 /**
 * Returns a pointer to the current FA.
 *
 * @return the current FA
 */
 public FA getFa() 
 {
  return this.fa;
 }

 /**
 * Sets the name of the FA to the given string.
 *
 * @param inputName the name of the FA
 */
 public void setFaName(String inputName) 
 {
  this.faName = inputName;
 }
 
 /**
 * Sets the type of FA 
 *
 * @param inputName the name of the FA
 */
 public static void setFaType(String inputType) 
 {  
  faType = inputType;  
 } 
 
 /**
 * @return if FA is a DFA
 *
 */
 public static String getFaType()
 {
  return faType;	
 }
 
 public static boolean isItDFA()
 {
  if(faType.equals("dfa"))
  {
   return true;
  } 			
  else
   return false; 		
 }
 
 /**
  * @return if the FA is an NFA
  *
  */
 public static boolean isItNFA()
 {
 
 	if(faType.equals("nfa"))
 	{
 		return true; 			
 	}
 	else 
	 	return false;
 }
 
 /**
  * @return if the FA is an eNFA
  *
  **/
 public static boolean isIteNFA()
 {
 
 	if(faType.equals("enfa"))
 	{
 		return true; 			
 	}
 	else 
	 	return false;
 }
 
 /**
  * @return if the FA type is not yet specified
  *
  **/ 
 public static boolean isItFA()
 { 	
 
 	if(faType.equals("fa"))
 	{
 		return true; 			
 	}
 	else 
	 	return false;
 } 

 /**
 * Checks if a mouse has been clicked on a location being occupied by
 * a state in the workspace panel. The location should be in a
 * circle representing the state.
 *
 * @param fa the current FA
 * @param e mouse event on the workspace panel
 *
 * @return the position of the state in the state list if such a state
 * occupies the clicked location; -1 if no state occupies
 * such location
 */
 public int getOccupiedState(FA fa, MouseEvent e) 
 {
  double d;
  for (int i = 0; i < fa.getStateCount(); i++) 
  {
   state = fa.getStateAt(i);
   d = Point.distance((double) state.getX(),
   (double) state.getY(), (double) e.getX(),
   (double) e.getY());
   if (d <= Radius) 
   {
    return i;
   }
  }
  return -1;
 }

 /**
 * Checks if a transition exists on the clicked point in the workspace
 * panel. The location should be inside a box where the transition
 * input character is displayed.
 *
 * @param fa the current FA
 * @param e mouse event
 *
 * @return the position of the transition in the transition list if
 * such a transition occupies the clicked location; -1 if no
 * transition occupies such location
 */
 public int getOccupiedTransition(FA fa, MouseEvent e) 
 {
  for (int i = 0; i < fa.getTransitionCount(); i++) 
  {
   Transition t = fa.getTransitionAt(i);
   if (t != null)
   {
    Point p = getTCoordinates(t.getSource(),t.getDestination());
    int tx = (int) p.getX() - 5;
    int ty = (int) p.getY() - 15;
    for (int j = 0; j < t.getIndex(); j++) 
    {
     tx += 15;
    }
    int x = e.getX();
    int y = e.getY();
    if ((x >= tx) && (x <= tx + 10) && (y >= ty) && (y <= ty + 20)) 
    {
     return i;
    }
   }
  }
  return -1;
 }

 /**
 * Sets the selected state to the given state.
 *
 * @param s the state to be selected
 */ 
 public void setSelectedState(State s) 
 {
  this.selectedState = s;
 }

 /**
 * Returns the selected state.
 *
 * @return the current selected state
 */

 public State getSelectedState() 
 {
  return this.selectedState;
 } 

 /**
 * Sets the selected transition to the given transition.
 *
 * @param t the transition to be selected
 */

 public void setSelectedTransition(Transition t) 
 {
  this.selectedTrans = t;
 }

 /**
 * Returns the current selected transition.
 *
 * @return the current selected transition
 */
 
 public Transition getSelectedTransition() 
 {
  return this.selectedTrans;
 }

 /**
 * Sets the name of the state to the given string.
 *
 * @param inputName the name for the state
 */
 public void setStateName(String inputName) 
 {
  this.stateName = inputName;
 }

 /**
 * Calculates for the coordinates of a transition line given the
 * source state's and the destination state's position in the
 * workspace panel. The points of the line are passed to ({@link
 * #srcX}, {@link #srcY}) and ({@link #dstX}, {@link #dstY}). This
 * method also calculates the point for the upper left corner of the
 * box where the transition input should be displayed (transition
 * box).
 *
 * @param source the source state
 * @param destination the destination state
 *
 * @return the point for the upper left corner of the transition box
 */
 public Point getTCoordinates(State source, State destination) 
 {
  int r = 25;
  int x1 = source.getX();
  int y1 = source.getY();
  int x2 = destination.getX();
  int y2 = destination.getY();
  double d = 0;
  double distance = Point.distance(x1, y1, x2, y2);
   /* possible transition source points */
  double[][] src =
  {
   { 
    x1 + (r / Math.sqrt(2)), y1 + (r / Math.sqrt(2)) 
   },
   { 
   x1 - (r / Math.sqrt(2)), y1 - (r / Math.sqrt(2)) 
   }
  };
  
  /* possible transition points */
  double[][] dst = 
  {
   { 
    x2 + (r / Math.sqrt(2)), y2 - (r / Math.sqrt(2)) 
   },
   { 
    x2 - (r / Math.sqrt(2)), y2 + (r / Math.sqrt(2)) 
   }
  };
  int i;
  int j;
 
  /* if transition loop on the same state, then return these
  * transition point
  */
  if (source == destination) 
  {
   srcX = x1 - 15;
   srcY = y1 - 52;
   return new Point(srcX + 5, srcY);
   /* if not, then get the shortest possible line for the
   * transition using the points in the src points and dst points
   * array
   */
  } 
  else 
  {
   for (i = 0; i < src.length; i++) 
   {
    for (j = 0; j < dst.length; j++) 
    {
     d = Point.distance(src[i][0], src[i][1], dst[j][0],
     dst[j][1]);
     if (d < distance) 
     {
      distance = d;
      srcX = (int) src[i][0];
      srcY = (int) src[i][1];
      dstX = (int) dst[j][0];
      dstY = (int) dst[j][1];
     }
    }
   }
   /* then return the points for the shortest transition line */
   return new Point(srcX + ((dstX - srcX) / 3),
   srcY + ((dstY - srcY) / 3));
  }
 }

 /**
 * Sets the input symbol of the transition.
 *
 * @param input the input symbol of the transition
 */
 public void setTransInput(String input) 
 {  	
  if((input.charAt(0)==92) && (input.charAt(1)=='e'))
  {
  	this.input = FAChecker.epsilonChar();
  }   
  else
  {
  	this.input = input.charAt(0);
  }  
 }

 /**
 * Checks if the clicked point is occupied by a state. If it is, the
 * current selected state is deselected, and the clicked state is
 * selected.
 *
 * @param e mouse event
 */

 public void checkStateSelect(MouseEvent e) 
 {
  if (selectedState != null) 
  {
   selectedState = null;
   repaint();
   container.setStatus("");
  }
  if (getOccupiedState(fa, e) >= 0) 
  {
   selectedState = fa.getStateAt(getOccupiedState(fa, e));
   reSelectState(selectedState, g);
   container.setStatus("State " + selectedState.getName());
  }
 }

 /**
 * Checks if the clicked point is occupied by a transition box. If it
 * is, the current selected transition box is deselected, and the
 * clicked transition box is selected.
 *
 * @param e mouse event
 */
 public void checkTransSelect(MouseEvent e) 
 {
  if (selectedTrans != null) 
  {
   selectedTrans = null;
   repaint();
   container.setStatus("");
  }
  if (getOccupiedTransition(fa, e) >= 0) 
  {
   selectedTrans = fa.getTransitionAt(getOccupiedTransition(fa, e));
   reSelectTransition(selectedTrans, g);
   container.setStatus("Transition from state "
   + selectedTrans.getSource().getName()
   + " to state "
   + selectedTrans.getDestination().getName()
   + " on input "
   + String.valueOf(selectedTrans.getInput()));
   }
  }

 /**
 * Deselects a state, and thus drawing it in normal form in the
 * workspace panel.
 *
 * @param state the state to be deselected
 * @param g the graphics component of the workspace panel
 */
 public void deSelectState(State state, Graphics g) 
 {
  Graphics2D g2d = (Graphics2D) g;
  g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
  RenderingHints.VALUE_ANTIALIAS_ON);
  g2d.setFont(new Font(g2d.getFont().getName(), Font.BOLD,
  g2d.getFont().getSize()));
  String stateName = state.getName();
  int nameLength = stateName.length();
  if (nameLength > 4) 
  {
   stateName = stateName.substring(0, 4) + "...";
   nameLength = 4;
  }
  if (state.getAcceptFlag()) 
  {
   g2d.setColor(blue_accept);
   g2d.fillOval(state.getX() - 25, state.getY() - 25, 50, 50);
   g2d.setColor(darkblue);
   g2d.fillOval(state.getX() - 23, state.getY() - 23, 46, 46);
   g2d.setColor(blue_accept);
   g2d.fillOval(state.getX() - 21, state.getY() - 21, 42, 42);
   g2d.setColor(darkblue);
   g2d.drawString(stateName, state.getX() - (nameLength * 6 / 2),
   state.getY());
  } 
  else 
  {
   g2d.setColor(lightblue);
   g2d.fillOval(state.getX() - 25, state.getY() - 25, 50, 50);
   g2d.setColor(darkblue);
   g2d.drawString(stateName, state.getX() - (nameLength * 6 / 2),
   state.getY());
  } 
  if (fa.getStartingState() == state) 
  {
   Polygon arrowhead = new Polygon();
   arrowhead.addPoint(state.getX() - 20, state.getY());
   arrowhead.addPoint(state.getX() - 30, state.getY() - 25);
   arrowhead.addPoint(state.getX() - 30, state.getY() + 25);
   g2d.setColor(lightgreen);
   g2d.fillPolygon(arrowhead);
   g2d.setColor(lightpink);
   g2d.drawPolygon(arrowhead);
  }
 }

 /**
 * Deselects the current selected transition. To properly draw all
 * transitions with the same path as the deselected transition, and
 * also the transition line, all transitions are redrawn. The source
 * and destination states of the deselected transition are also
 * redrawn. This method can be used to simply redraw the transitions
 * of the FA by setting the the transition parameter to null.
 *
 * @param t the transition to be deselected
 * @param g the graphics component of the workspace panel
 */
 public void deSelectTransition(Transition t, Graphics g) 
 {
  if (fa != null) 
  {
   /* redraw all transitions in normal form */
   for (int i = 0; i < fa.getTransitionCount(); i++) 
   {
    drawTransition(fa.getTransitionAt(i), false, g);
   }
   /* if a transition has been selected previously, then
   * redraw its source and destination states
   */
   if (transition != null) 
   {
    deSelectState(t.getSource(), g);
    deSelectState(t.getDestination(), g);
   }
  }
 }
 
 /**
 * Draws an arrowhead specified by the source points and destination
 * points.
 *
 * @param srcX x-coordinate of source point
 * @param srcY y-coordinate of source point
 * @param dstX x-coordinate of destination point
 * @param dstY y-coordinate of destination point
 * @param radius size of the arrowhead
 * @param g the graphics component of the workspace panel
 */
 public void drawArrowHead(int srcX, int srcY, int dstX, int dstY,
 int radius, Graphics g) 
 {
  Graphics2D g2d = (Graphics2D) g;
  int dY = dstY;
  int dB = srcY;
  int x1;
  int y1;
  int x2;
  int y2;
  double theta;
  double theta1;
  double theta2;
  double m = 0;
  Polygon arrowhead = new Polygon();
  Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
  int screenHeight = screenSize.height;
  int newdstY = screenHeight - dstY;
  int newsrcY = screenHeight - srcY;
  if (srcX != dstX) 
  {
   m = (double) ((newdstY - newsrcY) / (double) (dstX - srcX));
   theta = Math.atan(m);
   if (m >= 0) 
   {
    if (dstX > srcX) 
    {
     theta1 = 135.0f + Math.toDegrees(theta);
     theta2 = theta1 + 90.0f;
    } 
    else 
    {
     theta1 = 45.0f + Math.toDegrees(theta);
     theta2 = theta1 + 270.0f;
    }
   } 
   else 
   {
    if (dstX > srcX) 
    {
     theta1 = 135.0f + Math.toDegrees(theta);
     theta2 = theta1 + 90.0f;
    } 
    else 
    {
     theta1 = 45.0f + Math.toDegrees(theta);
     theta2 = theta1 + 270.0f;
    }
   }
  } 
  else 
  {
   if (newdstY > newsrcY) 
   {
    theta = 90.0f;
    theta1 = theta + 135.0f;
    theta2 = theta1 + 90.0f;
   } 
   else 
   {
    theta = 90.0f;
    theta1 = theta - 45.0f;
    theta2 = theta1 + 90.0f;
   }
  }
  x1 = (int) (radius * Math.cos(Math.toRadians(theta1)) + dstX);
  y1 = (int) (radius * Math.sin(Math.toRadians(theta1)) + newdstY);
  x2 = (int) (radius * Math.cos(Math.toRadians(theta2)) + dstX);
  y2 = (int) (radius * Math.sin(Math.toRadians(theta2)) + newdstY);
  y1 = screenHeight - y1;
  y2 = screenHeight - y2;
  arrowhead.addPoint(dstX, dstY);
  arrowhead.addPoint(x1, y1);
  arrowhead.addPoint(x2, y2);
  g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
  RenderingHints.VALUE_ANTIALIAS_ON);
  g2d.fillPolygon(arrowhead);
 }

 /**
 * Draws the transition box associated with the transition.
 *
 * @param x x-coordinate of upper left corner of transition box
 * @param y y-coordinate of upper left corner of transition box
 * @param index the transition index counter
 * @param input the transition input symbol
 * @param selectFlag determines whether the input box should be
 * highlighted (selected) or not
 * @param g the graphics component of the workspace panel
 */
 public void drawInputBox(int x, int y, int index, String input,
 boolean selectFlag, Graphics g) 
 {
  Graphics2D g2d = (Graphics2D) g;
  g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
  RenderingHints.VALUE_ANTIALIAS_ON);
  g2d.setFont(
  new Font(g2d.getFont().getName(), Font.BOLD,
  g2d.getFont().getSize()));
  for (int i = 0; i < index; i++) 
  {
   x += 15;
  }
  if (selectFlag == true) 
  {
   g2d.setColor(lightpink);
  } 
  else 
  {
   g2d.setColor(lightgreen);
  }
  g2d.fillRect(x - 5, y - 15, 15, 20);
  if (selectFlag == true) 
  {
   g2d.setColor(lightyellow);
  } 
  else 
  {
   g2d.setColor(darkblue);
  }
  g2d.drawRect(x - 5, y - 15, 15, 20);
  //if (FAChecker.isBackslashCharacter(input.charAt(0))) {
  // if (input.equals(" ")) {
  // input = "s";
  // }
  // input = "\\" + input;
  //}
  g2d.drawString(input, x, y);
 }

 /**
 * Draws the transition line associated with a transition.
 *
 * @param x1 x-coordinate of the source of the transition line
 * @param y1 y-coordinate of the source of the transition line
 * @param x2 x-coordinate of the destination of the transition line
 * @param y2 y-coordinate of the destination of the transition line
 * @param g the graphics component of the workspace panel
 */
 public void drawTLine(int x1, int y1, int x2, int y2, Graphics g) 
 {
  Graphics2D g2d = (Graphics2D) g;
  g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
  RenderingHints.VALUE_ANTIALIAS_ON);
  g2d.drawLine(x1, y1, x2, y2);
  drawArrowHead(x1, y1, x2, y2, 10, g);
 }

 /**
 * Draws a transition loop associated with a transition from a state
 * to the same state.
 *
 * @param x x-coordinate of upper left corner of rectangle containing
 * the transition loop
 * @param y y-coordinate of upper left corner of rectangle containing
 * the transition loop
 * @param g the graphics component of the workspace panel
 */
 public void drawTLoop(int x, int y, Graphics g) 
 {
  Graphics2D g2d = (Graphics2D) g;
  g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
  RenderingHints.VALUE_ANTIALIAS_ON);
  g2d.drawOval(x, y, 30, 30);
  drawArrowHead(x + 40, y + 10, x + 20, y + 30, 10, g);
 }

 /**
 * Draws the transition on the workspace panel.
 *
 * @param t the transition to be drawn
 * @param selectFlag determines whether the transition should be drawn
 * in normal form or in highlighted (selected) form
 * @param g the graphics component of the workspace panel
 */
 public void drawTransition(Transition t, boolean selectFlag,
 Graphics g) 
 {
  if (t != null) 
  {
   State source = t.getSource();
   State destination = t.getDestination();
   String input = String.valueOf(t.getInput());
   Point p = getTCoordinates(source, destination);
   if (selectFlag == true) 
   {
    g.setColor(lightpink);
   } 
   else 
   {
    g.setColor(Color.BLACK);
   }
   if ((t.getIndex() == 0) || (selectFlag == true)) 
   {
    if (source == destination) 
    {
     drawTLoop(srcX, srcY, g);
    } 
    else 
    {
     drawTLine(srcX, srcY, dstX, dstY, g);
    }
   }
   drawInputBox((int) p.getX(), (int) p.getY(), t.getIndex(),
   String.valueOf(t.getInput()), selectFlag, g);
  }
 }

 /**
 * Invoked when the mouse button has been clicked (pressed and
 * released) on a component. The workspace {@link #command} should be
 * set when invoking this method. The following are executed for each
 * command in the workspace panel: <CODE> NONE: This method checks
 * for a mouseclick on a state or transition and selects it. NEWFILE:
 * This method creates a new FA and sets the FA parameters to the
 * values contained in {@link #fa}Name and {@link #IA}. A starting
 * state is also added, using the values in {@link #stateName} and
 * {@link #accepting}. The state is then drawn into the workspace.
 * ADDSTATE: This method creates a new state with the name in {@link
 * #stateName} and accepting flag in {@link #accepting}, and then
 * draws this state. ADDTRANSITION: This method creates a new
 * transition with the source and destination states specified in
 * {@link #SrcState} and {@link #DstState} and the transition input i
 * {@link #input}. The transition is then drawn into the workspace.
 * </CODE>
 *
 * @param e MouseEvent
 */

 public void mouseClicked(MouseEvent e) 
 {
  if ((command == NONE) && (this.fa != null)) 
  {
   g = getGraphics();
   Radius = 25;
   checkStateSelect(e);
   checkTransSelect(e);
  } 
  else if (command == NEWFILE) 
  {
   g = getGraphics();
   Radius = 50;
   state = new State(stateName, accepting);
   state.setCoordinates(e.getX(), e.getY());
   accepting = false;
   fa = new FA(faName);
   fa.setAlphabet(IA);
   fa.addState(state);
   repaint();
   command = NONE;
   container.setStatus("");
  } 
  else if (command == ADDSTATE) 
  {
   g = getGraphics();
   Radius = 50;
   if (getOccupiedState(fa, e) == -1) 
   {
    state = new State(stateName);
    state.setCoordinates(e.getX(), e.getY());
    state.setAcceptFlag(accepting);
    accepting = false;
    fa.addState(state);
    repaint();
    command = NONE;
    container.setStatus("");
   }
  } 
  else if (command == ADDTRANSITION) 
  {
   g = getGraphics();
   checkStateSelect(e);
   if (selectedState != null) 
   {
    if (SrcState == null) 
    {
     SrcState = selectedState;
     if (isItDFA())
     { 
      if (!FAChecker.isValidTransition(container, fa,
      SrcState, input)) 
      {
       args[0] = SrcState.getName();
       args[1] = String.valueOf(input);
       SrcState = null;
       DstState = null;
       command = NONE;
       repaint();
      } 
      else 
      {
       container.setStatus("Select the destination "
       + "state for this transition...");
      }
     }
     else if(isItNFA())
     {
      container.setStatus("Select the destination "
      + "state for this transition of the NFA...");
     }
     else if(isIteNFA())
     {
      //input alphabet should accept epsilon
      container.setStatus("Select the destination "
      + "state for this transition of the e-NFA...");
     }
     else if(isItFA())
     {
      container.setStatus("Select the destination "
      + "state for this transition of the FA...");
     }
    } 
    else 
    {
     DstState = selectedState;
     Transition t = new Transition();
     t.setSource(SrcState);
     t.setDestination(DstState);
     t.setInput(input);
     fa.addTransition(t);
     drawTransition(t, false, g);
     SrcState = null;
     DstState = null;
     command = NONE;
     resetStateSelected();
     repaint();
    }
   }
  }
 }

 /**
 * Invoked when a mouse button is pressed on a component and then
 * dragged. This method ignores a right-click on the workspace, but
 * on a left click, if the mouse event starts on an existing state,
 * that state can be dragged around the workspace panel.
 *
 * @param e MouseEvent
 */
 public void mouseDragged(MouseEvent e) 
 {
  if ((fa != null) && (getOccupiedState(fa, e) >= 0)) 
  {
   if (e.isMetaDown())
   {
   } 
   else 
   {
    int NewX = e.getX();
    int NewY = e.getY();
    Dimension panelSize = this.getSize();
    int panelWidth = panelSize.width;
    int panelHeight = panelSize.height;
    if (e.getX() < 25) 
    {
     NewX = 25;
    }
    if (e.getX() > (panelWidth - 25)) 
    {
     NewX = panelWidth - 25;
    }
    if (e.getY() < 25) 
    {
     NewY = 25;
    }
    if (e.getY() > (panelHeight - 25)) 
    {
     NewY = panelHeight - 25;
    }
    state.setCoordinates(NewX, NewY);
    repaint();
   }
  }
 }

 /**
 * Invoked when the mouse enters a component.
 *
 * @param e MouseEvent
 */
 public void mouseEntered(MouseEvent e) 
 {
 }

 /**
 * Invoked when the mouse exits a component.
 *
 * @param e MouseEvent
 */
 public void mouseExited(MouseEvent e) 
 {
 }

 /**
 * Invoked when the mouse cursor has been moved onto a component but
 * no buttons have been pushed.
 *
 * @param e MouseEvent
 */
 public void mouseMoved(MouseEvent e)
 {
 }

 /**
 * Invoked when a mouse button has been pressed on a component.
 *
 * @param e MouseEvent
 */
 public void mousePressed(MouseEvent e) 
 {
 }

 /**
 * Invoked when a mouse button has been released on a component.
 *
 * @param e MouseEvent
 */
 public void mouseReleased(MouseEvent e) 
 {
 }

 /**
 * Paints each of the components in this container. Specifically, this
 * method paints the graphics components into the workspace panel.
 *
 * @param g the graphics component of the workspace panel
 */
 public void paintComponent(Graphics g) 
 {
  super.paintComponent(g);
  if (fa != null) 
  {
   if (fa.getTransitionCount() > 0) 
   {
    deSelectTransition(null, g);
    if (selectedTrans != null) 
    {
     reSelectTransition(selectedTrans, g);
    }
   }
   for (int i = 0; i < fa.getStateCount(); i++) 
   {
    state = fa.getStateAt(i);
    deSelectState(state, g);
    if (state == selectedState) 
    {
     reSelectState(state, g);
    }
   }
  }
 }

 /**
 * Reselects and draws in highlighted form the specified state.
 *
 * @param state the state to be reselected
 * @param g the graphics component of the workspace panel
 */
 public void reSelectState(State state, Graphics g) 
 {
  Graphics2D g2d = (Graphics2D) g;
  g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
  g2d.setFont(new Font(g2d.getFont().getName(), Font.BOLD,g2d.getFont().getSize()));
  String stateName = state.getName();
  int nameLength = stateName.length();
  if (nameLength > 4) 
  {
   stateName = stateName.substring(0, 4) + "..";
   nameLength = 4;
  }
  if (state.getAcceptFlag()) 
  {
   g2d.setColor(darkpink);
   g2d.fillOval(state.getX() - 25, state.getY() - 25, 50, 50);
   g2d.setColor(darkblue);
   g2d.fillOval(state.getX() - 23, state.getY() - 23, 46, 46);
   g2d.setColor(darkpink);
   g2d.fillOval(state.getX() - 21, state.getY() - 21, 42, 42);
   g2d.setColor(darkblue);
   g2d.drawString(stateName, state.getX() - (nameLength * 6 / 2),
   state.getY());
  }
  else 
  {
   g2d.setColor(darkpink);
   g2d.fillOval(state.getX() - 25, state.getY() - 25, 50, 50);
   g2d.setColor(darkblue);
   g2d.drawString(stateName, state.getX() - (nameLength * 6 / 2),
   state.getY());
  }
  if (fa.getStartingState() == state) 
  {
   Polygon arrowhead = new Polygon();
   arrowhead.addPoint(state.getX() - 20, state.getY());
   arrowhead.addPoint(state.getX() - 30, state.getY() - 25);
   arrowhead.addPoint(state.getX() - 30, state.getY() + 25);
   g2d.setColor(lightgreen);
   g2d.fillPolygon(arrowhead);
   g2d.setColor(lightpink);
   g2d.drawPolygon(arrowhead);
  }
 }

 /**
 * Reselects and draws in highlighted form the specified transition.
 *
 * @param t the transition to be selected
 * @param g the graphics component of the workspace panel
 */
 public void reSelectTransition(Transition t, Graphics g) 
 {
  drawTransition(t, true, g);
  deSelectState(t.getSource(), g);
  deSelectState(t.getDestination(), g);
 }

 /**
 * Resets the workspace panel, so the FA is reset to null, the
 * command is reset to NONE, and the workspace graphics component is
 * emptied.
 */
 public void reset() 
 {
  this.fa = null;
  this.command = NONE;
  this.repaint();
 }

 /**
 * Resets the current selected state, so that no state will be
 * selected.
 */
 public void resetStateSelected() 
 {
  g = getGraphics();
  if (selectedState != null) 
  {
   deSelectState(selectedState, g);
   this.selectedState = null;
  }
  container.setStatus("");
 }

 /**
 * Resets the current selected transition, so that no transition will
 * be selected.
 */
 public void resetTransSelected()
 {
  g = getGraphics();
  deSelectTransition(selectedTrans, g);
  this.selectedTrans = null;
  container.setStatus("");
 }
}
